#############################################################################
# Author:                                                                   #
# ------                                                                    #
#  Anton Kokalj                                  Email: Tone.Kokalj@ijs.si  #
#  Department of Physical and Organic Chemistry  Phone: x 386 1 477 3523    #
#  Jozef Stefan Institute                          Fax: x 386 1 477 3811    #
#  Jamova 39, SI-1000 Ljubljana                                             #
#  SLOVENIA                                                                 #
#                                                                           #
# Source: $XCRYSDEN_TOPDIR/Tcl/Viewer.tcl                                       
# ------                                                                    #
# Copyright (c) 1996-2003 by Anton Kokalj                                   #
#############################################################################

proc ViewMol {top {file {}} } {
    global light mode2D widsize maingeom fileselect style3D xcMisc system

    set top .
    wm title $top "XCrySDen"
    wm iconname $top "XCrySDen"
    wm geometry $top +100+100

    set fac1 $xcMisc(resolution_ratio1)
    set fac2  $xcMisc(resolution_ratio2)
    # INITIALIZATIONS
    # ---------------
    set widsize(conW) [expr round(130 * $fac2)]
    set widsize(menH) [expr round(35 * $fac1)]
    set widsize(meaH) [expr round(35 * $fac1)]
    #set widsize(conW) 130
    #set widsize(menH) 35 
    #set widsize(meaH) 35 

    # make a global widget arrangement, that is menu, mesawin, control & mea
    PlaceGlobWin 0 [expr round(650 * $fac1)] [expr round(650 * $fac1)]
    if [winfo exists .title] { 
	wm deiconify .
    }
    focus $top
    #tkwait visibility .mesa

    bind $top <Configure> {ResizeGlobWin}
    #---------------------------
    # THIS IS FOR MENU
    ViewMolMenu .menu .mesa
    bind .mesa <B1-Motion> { .mesa xc_B1motion %x %y }
    bind .mesa <B2-Motion> { .mesa xc_B2motion %x %y }
    bind .mesa <B1-ButtonRelease> { .mesa xc_Brelease B1 }
    bind .mesa <B2-ButtonRelease> { .mesa xc_Brelease B2 }
    #---------------------------

    #-----------------------#
    #    CONTROL FRAME      #
    #-----------------------#
    # Control Frame will consist from more frames, like:
    # special frame for DisplayModes, special frame for Translation, Rotation,
    # etc    
    set c [canvas .ctrl.c -yscrollcommand [list .ctrl.s set]]
    scrollbar .ctrl.s -width 8 -orient vertical -command [list .ctrl.c yview]
    pack .ctrl.s -side right -fill y
    pack .ctrl.c -side left -fill both
    # here get the requested width of scrollbar
    set widsize(scrW) [winfo reqwidth .ctrl.s]
    set widsize(canW) [expr round(130 * $fac2) - $widsize(scrW) + 1]
    # create FRAME to hold every LABEL&ENTRY
    set fc [frame .ctrl.c.f -bd 0 -width $widsize(canW)]
    $c create window 0 0 -anchor nw -window $fc -width $widsize(canW) \
	    -tags frame

    # define vorious frames for above reason
    set f1 [frame $fc.fr1 -relief raised -bd 2]
    set f2 [frame $fc.fr2 -relief raised -bd 2]
    set f3 [frame $fc.fr3 -relief raised -bd 2]
    # miscellaneous images
    image create photo ak -format gif -file $system(BMPDIR)/xcrysden_big.gif
    set fdummy [button $fc.fdummy -image ak -relief ridge -bd 4 \
    	    -anchor center -height 100 -command xcAbout]

    ##########################################################################
    # FRAME #1 for translation
    ViewMolFrame1 $f1

    ##########################################################################
    # FRAME #2 for rotation
    ViewMolFrame2 $f2

    ###########################################################################
    # now pack #f1 $f2 #f3 #f4
    pack $f1 $f2 -side top -fill x -padx 0 -pady 0 \
	    -ipadx 2 -ipady 2
    pack $f3 -side top -fill both -expand 1 -padx 0 -pady 0 \
	    -ipadx 2 -ipady 2
    ##################
    # DUMMY FRAME
    pack $fdummy -side bottom -fill both -expand 1
    
    ###########################################################################
    # FRAME #3 for DISPLAYMODES
    # do we want Shades (3D or 2D DisplayModes)
    set f31 [frame $f3.1]
    set f32 [frame $f3.2]
    set light On
    # canvas $f3.canv -width $widsize(conW) -height 150 
    canvas $f3.canv 
    pack $f31 $f32 $f3.canv -side top -fill both -expand 1
    tkwait visibility $f3.canv    
    RadioButCmd $f31 "Lighting:" light Lighting top left 1 1 5 "On" "Off"
    Lighting On
    #$f3.canv configure -height [winfo reqwidth $f3.canv]

    #-----------------------------------------#
    #     CONTROL FRAME - END - END - END     #
    #-----------------------------------------#

    #---------------#
    #  CLOSE FRAME  #
    #---------------#
    set maingeom "Maxi"
    #set b0 [button .close.0 -text F -width 1 -command [list MainGeom F]]
    #$b0 config -state disabled
    set b1 [button .close.1 -textvariable maingeom -width 3 -command MainGeom]
    set b2 [button .close.2 -text "Exit" -width 3 -command exit_pr]
    pack $b1 $b2 -side left -padx 0 -pady 0 -ipadx 0 -ipady 0 -expand 1 -fill x

    set mode2D(WF) On
    set mode2D(PL) Off
    set mode2D(BS1) Off
    set mode2D(BS2) Off

    #-------------------------#
    #      MEASURE FRAME      #
    #-------------------------#
    set meaF [frame .mea.f1 -relief ridge -bd 4]
    set atmi [button $meaF.ainf \
	    -highlightthickness 0 -text "AtomInfo" -command \
	    [list PreSel .sel_dist .mesa "Atom Info" \
	    "For \"Atom Info\" click on one atom" AtomInfo 1]]
    set flbd [button $meaF.dis \
	    -highlightthickness 0 -text "Distance" -command \
	    [list PreSel .sel_dist .mesa "Distance" \
	    "For \"Distance\" click on two atoms" Distance 2]]
    set flba [button $meaF.ang \
	    -highlightthickness 0 -text "Angle" -command \
	    [list PreSel .sel_dist .mesa "Angle" \
	    "For \"Angle\" click on three atoms" Angle 3]]
    set flbdi [button $meaF.dih \
	    -highlightthickness 0 -text "Dihedral" -command \
	    [list PreSel .sel_dist .mesa "Dihedral" \
	    "For \"Dihedral\" click on four atoms" Dihedral 4]]
    set flbp [button $meaF.pln \
	    -highlightthickness 0 -text "Plane" -command \
	    [list PreSel .sel_plane .mesa "Plane" \
	    "For \"Plane\" click on three atoms" Plane 3]]
    $flbp config -state disabled

    set style3D(draw)   "solid"
    set style3D(shade)  "smooth"
    set style3D(point)  "1"
    set modl [frame .mea.f2 -relief raised -bd 2]
    set styl [label $modl.l1 -text "DrawStyle:" -relief flat -bd 0]
    set sty [menubutton $modl.sty \
	    -textvariable style3D(draw) \
	    -menu $modl.sty.menu \
	    -indicatoron 1 \
	    -relief raised]
    set menu1 [menu $modl.sty.menu -tearoff 0]
    $menu1 add command -label "solid" -command [list Style3D draw solid]
    $menu1 add command -label "wire"  -command [list Style3D draw wire]
    set shdl [label $modl.l2 -text "Shade:" -relief flat -bd 0]
    set shd [menubutton $modl.shd \
	    -textvariable style3D(shade) \
	    -menu $modl.shd.menu \
	    -indicatoron 1 \
	    -relief raised]
    set menu2 [menu $modl.shd.menu -tearoff 0]
    $menu2 add command -label "smooth" -command [list Style3D shade smooth]
    $menu2 add command -label "flat" -command [list Style3D shade flat]
    #set poil [label $modl.l3 -text "PointSize:" -relief flat -bd 0]
    #set poi [menubutton $modl.poi \
    #	     -textvariable style3D(point) \
    #	     -menu $modl.poi.menu \
    #	     -indicatoron 1 \
    #	     -relief raised]
    #set menu3 [menu $modl.poi.menu]
    #$menu3 add command -label "1" -command [list Style3D point 1]
    #$menu3 add command -label "2" -command [list Style3D point 2]
    #$menu3 add command -label "3" -command [list Style3D point 3]
    pack $meaF -side left -fill y
    pack $modl -expand 1 -fill both -side left
    pack $atmi $flbd $flba $flbdi $flbp \
	    $styl $sty $shdl $shd -side left -padx 0 

    # before opening a structure, a xc_mesacontext must be made !!!!!    
    # xc_mesacontext .mesa

    #--------------------------------------#
    # configure control canvas & scrollbar #
    #--------------------------------------#
    update
    set width  [expr [winfo width .ctrl] - $widsize(scrW)]
    set height [winfo height .ctrl]
    set h      [winfo reqheight $fc]
    $c config -width $width -height $height \
	    -scrollregion "0 0 $width $h"     
    #----------------------------------------------------#
    # END -- configure control canvas & scrollbar -- END #
    #----------------------------------------------------#
    if { $file != {} } {
	# open file and display it !!!
	OpenStruct .mesa $file
    }

    # disable all neccessary widgets
    xcGetState
    if [winfo exists .title] { 
	#after 500
	destroy .title 
    }
}


proc ResizeGlobWin {} {
    global w h
    if ![info exists w] { set w 0 }
    if ![info exists h] { set h 0 }
    # update only if size of "toplevel ." has changed
    if { $w != [winfo width .] || $h != [winfo height  .] } {
	puts stdout "ResizeGlobWin>>> RESIZE\n"
	flush stdout
	set w [winfo width .]
	set h [winfo height  .]
	PlaceGlobWin 1 $w $h
	update
	ConfigControlWid
    }
}


proc PlaceGlobWin {type w h} {
    global widsize unmapWin
    # type == 0 initial arrangement
    # type == 1 <Configure> rearrangement

    if { $type == 0 } {
	# this is for toplevel width & height     
	set w [expr $w + $widsize(conW)]
	set h [expr $h + $widsize(menH) + $widsize(meaH)]
	. configure -width $w -height $h
	#tkwait visibility .    
    }
    
    # on TOP will be MENU; on left xc_mesawin -> that is a custom widget
    # made for XCrySDen;
    # on right will be frame, that will hold everything --> this frame's name
    # will be control !!!
    # On BOTTOM there will be a MEASURE frame -> this frame's name will
    # be "mea"
    # On RIGHT BOTTOM there will be another frame that will hold Close button 

    # some calculation for PLACEing the three widged mentioned above
    # frame 
    set controlW $widsize(conW)
    set controlH [expr $h - $widsize(meaH)]
    set controlX [expr $w - $controlW]
    set controlY 0
    set widsize(conH) $controlH

    set menuX 0
    set menuY 0
    set menuW [expr $w - $controlW]
    #set menuW $w
    set menuH $widsize(menH)
    
    #new
    #set controlH [expr $controlH - $menuH]
    #set controlY $menuH
    #/
    
    set meaH $widsize(meaH)
    set meaW $menuW

    set mesaX 0
    set mesaY $menuH
    set mesaW [expr $w - $controlW]
    set mesaH [expr $h - $menuH - $meaH]

    set meaX 0
    set meaY [expr $menuH + $mesaH]

    set closeX [expr $w - $controlW]
    set closeY $meaY
    set closeW $controlW
    set closeH $meaH

    puts stdout "meaY=$meaY menuH=$menuH mesaH=$mesaH"
    flush stdout
        
    # initial arrangement of GlobWid
    if { $type == 0 } {
	# definition of above widgets
	set menuWid  [frame .menu -relief raised -bd 2 -class MenuBar]
	#set mesaWid  [xc_mesawin .mesa -bg "#000"]
	set mesaWid [togl .mesa \
		-width          400 \
		-height         400 \
		-ident          .mesa \
		-rgba           true  \
		-redsize        1     \
		-greensize      1     \
		-bluesize       1     \
		-double         true  \
		-depth          true  \
		-depthsize      1     \
		-accum          false \
		-accumredsize   1     \
		-accumgreensize 1     \
		-accumbluesize  1     \
		-accumalphasize 1     \
		-alpha          false \
		-alphasize      1     \
		-stencil        false \
		-stencilsize    1     \
		-auxbuffers     0     \
		-overlay        false \
		-stereo         false \
		-time           500   ]
	set conWid   [frame .ctrl  -relief flat   -bd 0 -class Viewer]
	set meaWid   [frame .mea   -relief flat   -bd 0 -class Viewer]
	set closeWid [frame .close -relief raised -bd 2 -class Viewer]

	set unmapWin(frame,main) [frame .mesa.unmap -bg "#000"]

	# place the above widgets
	place $menuWid -x $menuX -y $menuY \
		-width $menuW -height $menuH -anchor nw
	place $mesaWid -x $mesaX -y $mesaY \
		-width $mesaW -height $mesaH -anchor nw
	place $conWid -x $controlX -y $controlY \
		-width $controlW -height $controlH -anchor nw
	place $meaWid -x $meaX -y $meaY \
		-width $meaW -height $meaH -anchor nw
	place $closeWid -x $closeX -y $closeY \
		-width $closeW -height $closeH -anchor nw

	pack $unmapWin(frame,main) -anchor sw -side bottom
	set unmapWin(packinfo,$unmapWin(frame,main)) \
		[pack info $unmapWin(frame,main)]
    } else {
	# configure above widgets
	place configure .menu -x $menuX -y $menuY \
		-width $menuW -height $menuH -anchor nw
	place configure .mesa -x $mesaX -y $mesaY \
		-width $mesaW -height $mesaH -anchor nw
	place configure .ctrl -x $controlX -y $controlY \
		-width $controlW -height $controlH -anchor nw
	place configure .mea -x $meaX -y $meaY \
		-width $meaW -height $meaH -anchor nw
	place configure .close -x $closeX -y $closeY \
		-width $closeW -height $closeH -anchor nw

	#place configure $unmapWin(frame,main) -width 100 -height 100 \
	#	-x 0 -y $mesaH -anchor sw
    }
    return
}


proc ViewMolFrame1 {f} {
    global translstep system xcMisc widsize mody

    set fac1 $xcMisc(resolution_ratio1)
    set fac2 $xcMisc(resolution_ratio2)
 
    #initial value of translstep
    set translstep 0.05
    # make a frame that will hold UP, DOWN, LEFT & RIGHT button
    set w [expr round(122 * $fac2) - $widsize(scrW)]
    set h [expr round(122 * $fac2) - $widsize(scrW)]
    # set w [expr round(122 * $fac2)]
    # set h [expr round(122 * $fac2)]
    set cpad [expr round(4 * $fac1)]
    set ff [frame $f.1 -relief groove -bd 2 -width $w -height $h]
    pack $ff -side top -expand 1 -padx $cpad -pady $cpad 
    
    # create images up, down, left, right
    set b  [expr round(26 * $fac2)]; #this is button size
    image create photo up    -format gif \
	    -file $system(BMPDIR)/up1.gif -width $b -height $b
    image create photo down  -format gif \
	    -file $system(BMPDIR)/down1.gif -width $b -height $b
    image create photo left  -format gif \
	    -file $system(BMPDIR)/left1.gif -width $b -height $b
    image create photo right -format gif \
	    -file $system(BMPDIR)/right1.gif -width $b -height $b
    image create photo center -format gif \
	     -file $system(BMPDIR)/center_nice.gif -width $b -height $b
    # add something to $b bacause of button's borders 
    set b [expr $b + 8]
    set b1 [button $ff.b1 -image up \
	    -anchor center -width $b -height $b]
    set b2 [button $ff.b2 -image down \
	    -anchor center -width $b -height $b]
    set b3 [button $ff.b3 -image left \
	    -anchor center -width $b -height $b]
    set b4 [button $ff.b4 -image right \
	    -anchor center -width $b -height $b]
    set b5 [button $ff.b5 -image center \
	    -anchor center -width $b -height $b]
    
    # now place the buttons
    set fw [expr $w / 2 - round(4 * $fac1)]
    set fh [expr $h / 2 - round(2 * $fac1)]
    set offset 0

    place $b1 -x $fw -y [expr $fh - ( $b + $offset )] \
	    -anchor center -width $b -height $b
    place $b2 -x $fw -y [expr $fh + ( $b + $offset )] \
	    -anchor center -width $b -height $b
    place $b3 -x [expr $fw - ( $b + $offset )] -y $fh \
	    -anchor center -width $b -height $b
    place $b4 -x [expr $fw + ( $b + $offset )] -y $fh \
	    -anchor center -width $b -height $b
    place $b5 -x $fw -y $fh -anchor center -width $b -height $b

    bind $b1 <ButtonPress-1> [list Transl $b1 +y] 
    bind $b2 <ButtonPress-1> [list Transl $b2 -y] 
    bind $b3 <ButtonPress-1> [list Transl $b3 -x] 
    bind $b4 <ButtonPress-1> [list Transl $b4 +x] 
    bind $b5 <ButtonPress-1> {
	xc_newvalue .mesa $mody(L_TR_XTRANSL) 0.0
	xc_newvalue .mesa $mody(L_TR_YTRANSL) 0.0
    }
    bind $b1 <ButtonRelease-1> RelB1 
    bind $b2 <ButtonRelease-1> RelB1 
    bind $b3 <ButtonRelease-1> RelB1 
    bind $b4 <ButtonRelease-1> RelB1 
    bind $b5 <ButtonRelease-1> RelB1 
    
        
    ###############
    # Zoom + Zoom -
    set zoom1 [button $f.zoom1 -text "Zoom +" -width 4 ]
    set zoom2 [button $f.zoom2 -text "Zoom -" -width 4 ]
    bind $zoom1 <ButtonPress-1> [list Transl $zoom1 +z] 
    bind $zoom2 <ButtonPress-1> [list Transl $zoom2 -z]
    bind $zoom1 <ButtonRelease-1> [list RelB1] 
    bind $zoom2 <ButtonRelease-1> [list RelB1]
    
    ######################
    # SCALE for translstep
    set sc [scale $f.scale -from 0 -to 0.55 \
	    -length [expr round(115 * $fac2) - $widsize(scrW)] \
	    -variable translstep \
	    -orient horizontal -label "Trans Step:" -tickinterval 0.25 \
	    -digits 2 -resolution 0.05 -showvalue true \
	    -width [expr round(10 * $fac2)]]
    pack $sc -side bottom
    pack $zoom1 $zoom2 -side left -expand 1 
    
    # check if we need to adjust the width of .ctrl frame
    set w_zoom [expr [winfo reqwidth $zoom1] * 2]
    if { $w_zoom > $w } {
	set widsize(conW) [expr $w_zoom + $widsize(scrW) + 4]
	set w [winfo width .]
	set h [winfo height  .]
	PlaceGlobWin 1 $w $h
	.ctrl.c itemconfigure frame -width [expr $widsize(conW) - $widsize(scrW) + 1]
    }     
}


proc ViewMolFrame2 {f} {
    global rotstep downB1 xcMisc system

    #initial value of rotstep
    set rotstep 10

    #############
    # FRAME3
    set fac2 $xcMisc(resolution_ratio2)
    set w [expr round(4 * $fac2)]
    set f1 [frame $f.1 -relief flat -bd 0]
    set rx1 [button $f1.rx1 -text "Rot +X" -width $w -highlightthickness 1]
    set rx2 [button $f1.rx2 -text "Rot -X" -width $w -highlightthickness 1]
    bind $rx1 <ButtonPress-1> [list Rotate $rx1 +x] 
    bind $rx2 <ButtonPress-1> [list Rotate $rx2 -x]
    bind $rx1 <ButtonRelease-1> [list RelB1]
    bind $rx2 <ButtonRelease-1> [list RelB1]

    set f2 [frame $f.2 -relief flat -bd 0]
    set ry1 [button $f2.ry1 -text "Rot +Y" -width $w -highlightthickness 1] 
    set ry2 [button $f2.ry2 -text "Rot -Y" -width $w -highlightthickness 1] 
    bind $ry1 <ButtonPress-1> [list Rotate $ry1 +y]    
    bind $ry2 <ButtonPress-1> [list Rotate $ry2 -y]
    bind $ry1 <ButtonRelease-1> [list RelB1]
    bind $ry2 <ButtonRelease-1> [list RelB1]

    set f3 [frame $f.3 -relief flat -bd 0]
    set rz1 [button $f3.rz1 -text "Rot +Z" -width $w -highlightthickness 1] 
    set rz2 [button $f3.rz2 -text "Rot -Z" -width $w -highlightthickness 1] 
    bind $rz1 <ButtonPress-1> [list Rotate $rz1 +z]
    bind $rz2 <ButtonPress-1> [list Rotate $rz2 -z]
    bind $rz1 <ButtonRelease-1> [list RelB1]
    bind $rz2 <ButtonRelease-1> [list RelB1]

    set f4 [frame $f.4 -bd 2 -relief groove]
    set sc [scale $f.4.scale -from -180 -to 180 \
	    -length [expr round(115 * $fac2)] -variable rotstep \
	    -orient horizontal  -tickinterval 180 \
	    -digits 3 -resolution 1 -showvalue true \
	    -width [expr round(10 * $fac2)]]
    set rotlab [label $f4.lab -text "Rotation\nStep:" -justify right]
    set rotentry [entry $f4.entry -relief sunken \
	    -textvariable rotstep \
	    -width [expr round(5 * $fac2)]]
    pack $sc -side bottom 
    pack $rotlab $rotentry -side left -expand 1 -padx 0 -pady 0
    pack $f1 $f2 $f3 -side top -fill x -expand 1 -pady 1
    set i5 [expr round(5 * $fac2)]
    set i15 [expr round(15 * $fac2)]

    set f5 [frame $f.5 -bd 0 -relief flat]
    # create images up, down, left, right
    set b  [expr round(26 * $fac2)]; #this is button size
    image create photo rotXY    -format gif \
	    -file $system(BMPDIR)/rotXY1.gif -width $b -height $b
    image create photo rotXZ  -format gif \
	    -file $system(BMPDIR)/rotXZ1.gif -width $b -height $b
    image create photo rotYZ  -format gif \
	    -file $system(BMPDIR)/rotYZ1.gif -width $b -height $b
    image create photo rotAB    -format gif \
	    -file $system(BMPDIR)/rotAB.gif -width $b -height $b
    image create photo rotAC  -format gif \
	    -file $system(BMPDIR)/rotAC.gif -width $b -height $b
    image create photo rotBC  -format gif \
	    -file $system(BMPDIR)/rotBC.gif -width $b -height $b
    foreach {row col bt im com} [list \
	    0 0 $f5.xy rotXY {.mesa xc_rotate xy 0.0} \
	    0 1 $f5.yz rotYZ {.mesa xc_rotate yz 0.0} \
	    0 2 $f5.xz rotXZ {.mesa xc_rotate xz 0.0} \
	    1 0 $f5.ac rotAC {.mesa xc_rotate ac 0.0} \
	    1 1 $f5.ab rotAB {.mesa xc_rotate ab 0.0} \
	    1 2 $f5.bc rotBC {.mesa xc_rotate bc 0.0}] {
	button $bt -image $im -command $com -anchor center -width $b -height $b
	grid $bt -row $row -column $col -padx 1 -pady 1
    }
    pack $rx1 $rx2 $ry1 $ry2 $rz1 $rz2 -side left  \
	    -expand 1 -padx 0 -pady 0 -ipadx 0 -ipady 0
    pack $f5 -side bottom  -expand 1 -fill x -pady 2 -padx 1
    pack $f4 -side bottom  -expand 1 -pady $i5 -padx $i5 \
	    -ipadx $i15 -ipady $i5
}


proc Rotate {b dir} {
    global rotstep B1down err

    $b configure -relief sunken
    set B1down 1
    
    # check if $rotstep is real number
    number rotstep real
    if { $err == 1 } { 
	return
    }
    # everithing is OK !!!
    while { $B1down } {
	.mesa xc_rotate $dir $rotstep
	update 	
    }        
    $b configure -relief raised
    
    return -code break
}


proc Transl {b dir} {
    global translstep B1down

    $b configure -relief sunken
    set B1down 1

    # everithing is OK !!!
    while { $B1down } {
	.mesa xc_translate $dir $translstep
	update 	
    }
    $b configure -relief raised
    
    return -code break
}


proc RelB1 {} {
    global B1down
    
    set B1down 0
}


#
# arguments:
#      F  ... fullscreen
proc MainGeom { {F {}} } {
    global oldgeom maingeom global xcMisc

    # go to Maxi
    if { $maingeom == "Maxi" } {
	set maingeom "Mini"
	#set oldgeom [wm geometry .]
	set oldgeom [winfo geometry .]
	#set w [winfo vrootwidth .]
	#set h [winfo vrootheight .]
	set w [winfo screenwidth  .]
	set h [winfo screenheight .]
	###############################
	# testing "FULLSCREEN" option #
	###############################
	if { $F != {} } {
	    wm withdraw .
	    wm overrideredirect . true
	    update
	    wm deiconify .
	}
	###############################
	wm geometry . ${w}x${h}+$xcMisc(wm_rootXshift)+$xcMisc(wm_rootYshift)
    } else {
	set maingeom "Maxi"
	###############################
	# testing "FULLSCREEN" option #
	###############################
	if { $F != {} } {
	    wm withdraw .
	    wm overrideredirect . false
	    update
	    wm deiconify .
	}
	###############################
	wm geometry . $oldgeom
    }
}



proc Style3D {what how} {
    global style3D

    if {$what == "draw"} {
	set style3D(draw) $how
	xc_drawStyle3D .mesa $how 
    } 
    if {$what == "shade"} {
	set style3D(shade) $how
	xc_shadeModel3D .mesa $how
    }
    if {$what == "point"} {
	set style3D(point) $how
	puts stdout "style3D(point)"
	xc_pointSize .mesa $how
    }
}
