#############################################################################
# Author:                                                                   #
# ------                                                                    #
#  Anton Kokalj                                  Email: Tone.Kokalj@ijs.si  #
#  Department of Physical and Organic Chemistry  Phone: x 386 1 477 3523    #
#  Jozef Stefan Institute                          Fax: x 386 1 477 3811    #
#  Jamova 39, SI-1000 Ljubljana                                             #
#  SLOVENIA                                                                 #
#                                                                           #
# Source: $XCRYSDEN_TOPDIR/Tcl/dispModes.tcl                                       
# ------                                                                    #
# Copyright (c) 1996-2003 by Anton Kokalj                                   #
#############################################################################

proc Lighting {var} {
    global dispmode xcFonts xcMisc
    # var -- can be On ro Off
    #        On  - 3D displayModes
    #        Off - 2D displaymodes
    if { ![info exists dispmode(mode3D_ModeFrame)] } {
	set dispmode(mode3D_ModeFrame) {}
    }
    puts stdout "var = $var"
    if { $var == "Off" } {
	# if 3D widgets exists delete them
	if {[winfo exists .ctrl.c.f.fr3.canv.mode3D]} { 
	    destroy .ctrl.c.f.fr3.canv.mode3D
	}
	if {[winfo exists $dispmode(mode3D_ModeFrame)]} {
	    #xcDisableAll $dispmode(mode3D_ModeFrame)
	    destroy $dispmode(mode3D_ModeFrame)
	}
	    
	# if 2D widgets already exists, just return silently
	if {[winfo exists .ctrl.c.f.fr3.canv.mode2D]} { return }

	set f [frame .ctrl.c.f.fr3.canv.mode2D -bd 0]
	.ctrl.c.f.fr3.canv create window 0 0 \
	    -tags frame -anchor nw \
	    -width [winfo width .ctrl.c.f.fr3.canv] \
	    -height [winfo height .ctrl.c.f.fr3.canv] -window $f
	set b1  [button $f.wire -text "WireFrame" \
		-command [list Display2D WF]]
	set b2  [button $f.point -text "PointLine" \
		-command [list Display2D PL]]
	set b2a [button $f.pipe -text "Pipe&Ball" \
		-command [list Display2D PB]]
	set b3  [button $f.ball1 -text "BallStick-1" \
		-command [list Display2D BS1]] 
	set b4  [button $f.ball2 -text "BallStick-2" \
		-command [list Display2D BS2]]
	set b5  [button $f.fill  -text "SpaceFill" \
		-command [list Display2D SF]]
	set i15 [expr round(15 * $xcMisc(resolution_ratio1))]
	pack $b1 $b2 $b2a $b3 $b4 $b5 -side top -padx $i15 -pady 1 -fill x
	#new
	set _h [expr [winfo reqheight $b1] * 6 + 10]
	.ctrl.c.f.fr3.canv configure -height $_h
	.ctrl.c.f.fr3.canv itemconfigure frame -height  $_h
	#/
	
	bind $b4 <Visibility> ConfigControlWid 
	From3Dto2D    	 
    } elseif { $var == "On" } {
	# if 2D widgets exists delete them
	if [winfo exists .ctrl.c.f.fr3.canv.mode2D] { 
	    destroy .ctrl.c.f.fr3.canv.mode2D 
	}
	# if 3D widgets already exists, just return silently
	if {[winfo exists .ctrl.c.f.fr3.canv.mode3D]} { return }

	set f [frame .ctrl.c.f.fr3.canv.mode3D -bd 0]
	.ctrl.c.f.fr3.canv create window 0 0 \
	    -tags frame -anchor nw \
	    -width [winfo width .ctrl.c.f.fr3.canv] \
	    -height [winfo height .ctrl.c.f.fr3.canv] -window $f 
	
	if ![info exists dispmode(mode3D)] {
	    set dispmode(mode3D) "Over"
	}
	global radio_but_cmd_frame
	#if { ![winfo exists $dispmode(mode3D_ModeFrame)] } {
	#    ;
	#} else {
	#    xcEnableAll $dispmode(mode3D_ModeFrame)
	#}
	set rbs [RadioButCmd .ctrl.c.f.fr3.2 "Mode:" dispmode(mode3D) \
		     Mode3D top left 1 1 5 "Over" "Logic"]
	set dispmode(mode3D_ModeFrame) $radio_but_cmd_frame
	set rb1 [lindex $rbs 1]
	set rb2 [lindex $rbs 2]
	$rb1 configure -font $xcFonts(small)
	$rb2 configure -font $xcFonts(small)

	# display widgets for $dispmode(mode3D)
	Mode3D $dispmode(mode3D)	
	# what displayMode3D to render
	From2Dto3D
    } else {
	puts stderr "invalid value \"$var\" submitted to proc DispMode, must be one of On, Off"
	flush stderr
    }
    return
}


#==============================================================
# instead of xc_displayMode2D this proc should be used !!!!!!!!
#            ^^^^^^^^^^^^^^^^
proc Display2D {mode} {
    global mode2D

    set mode2D(WF)  Off
    set mode2D(PL)  Off
    set mode2D(PB)  Off
    set mode2D(BS1) Off
    set mode2D(BS2) Off
    set mode2D(SF)  Off

    if { $mode == "WF" } {
	set mode2D(WF) On
	xc_displayMode2D .mesa WF
    } elseif { $mode == "PL" } {
	set mode2D(PL) On
	xc_displayMode2D .mesa PL
    } elseif { $mode == "PB" } {
	set mode2D(PB) On
	xc_displayMode2D .mesa PB
    } elseif { $mode == "BS1" } {
	set mode2D(BS1) On
	xc_displayMode2D .mesa BS1
    } elseif { $mode == "BS2" } {
	set mode2D(BS2) On
	xc_displayMode2D .mesa BS2
    } elseif { $mode == "SF" } {
	set mode2D(SF) On
	xc_displayMode2D .mesa SF
    } else {
	puts stderr "invalid value \"$mode\" submitted to proc Display2D, must be one of WF, PL, PB, BS1, BS2, SF"
	flush stderr
    }
    return
}


proc Mode3D {mode} {
    global mode3D xcMisc

    set f .ctrl.c.f.fr3.canv.mode3D
    
    if { $mode == "Over" } {
	# if "Boolean/Logic" buttons CheckButtons exists, delete them
	if [winfo exists $f.bool] { 
	    destroy $f.bool
	}
	if [winfo exists $f.over1] {
	    return
	}
	set b1  [button $f.over1 -text "Stick" \
		-command [list DisplayOver3D S]]
	set b1a [button $f.over1a -text "Pipe&Ball" \
		-command [list DisplayOver3D PB]]
	set b2  [button $f.over2 -text "BallStick" \
		-command [list DisplayOver3D BS]]
	set b3  [button $f.over3 -text "SpaceFill" \
		-command [list DisplayOver3D SF]] 
	set i15 [expr round(15 * $xcMisc(resolution_ratio1))]
	pack $b1 $b1a $b2 $b3 -side top -padx $i15 -pady 1 -fill x \
		-ipadx 0 -ipady 0
	bind $b3 <Visibility> ConfigControlWid 
	#new
	set _h [expr [winfo reqheight $b1] * 4 + 10]
	.ctrl.c.f.fr3.canv configure -height $_h
	.ctrl.c.f.fr3.canv itemconfigure frame -height  $_h
	#/

    } elseif { $mode == "Logic" } {
	# Logic stands for Boolean
	if [winfo exists $f.over1] { 
	    destroy $f.over1 $f.over1a $f.over2 $f.over3
	}
	# create frame where CheckButtons will be displayed
	set ff [frame $f.bool]
	pack $ff -side top
	if ![info exists mode3D(pipe)]   { set mode3D(pipe) "Off" }
	if ![info exists mode3D(sticks)] { set mode3D(sticks) "Off" }
	if ![info exists mode3D(balls)]  { set mode3D(balls) "Off" }
	if ![info exists mode3D(space)]  { set mode3D(space) "Off" }
	
	set f1 [frame $ff.1]
	set f2 [frame $ff.2]
	set f3 [frame $ff.3]
	checkbutton $f1.cb -variable mode3D(sticks) \
	    -onvalue On -offvalue Off -text "STICK" \
	    -command [list DispBool3D mode3D(sticks)] -anchor sw 

	checkbutton $f2.cb -variable mode3D(balls) \
	    -onvalue On -offvalue Off -text "BALL" \
	    -command [list DispBool3D mode3D(balls)] -anchor sw 

	checkbutton $f3.cb -variable mode3D(space) \
	    -onvalue On -offvalue Off -text "SPACEFILL" \
	    -command [list DispBool3D mode3D(space)] -anchor sw 

	pack $f1.cb $f2.cb $f3.cb -side top -fill both \
	    -padx 0 -pady 0 -ipadx 0 -ipady 0
	pack $f1 $f2 $f3 -side top -fill x -expand 1
	bind $f3 <Visibility> ConfigControlWid 
	#new
	set _h [expr [winfo reqheight $f1.cb] * 3 + 10]
	.ctrl.c.f.fr3.canv configure -height $_h
	.ctrl.c.f.fr3.canv itemconfigure frame -height  $_h
	#/

    } else {
	puts stderr "invalid value \"$mode\" submitted to proc Mode3D, must be one of Over, Boolean"
	flush stderr
    }
    return
}


# ============================================================================
# istead of "xc_displayMode3D over" this proc should be used
#            ^^^^^^^^^^^^^^^^^^^^^
proc DisplayOver3D {mode} {
    global mode3D

    set mode3D(sticks) Off
    set mode3D(balls)  Off
    set mode3D(space)  Off
    
    if { $mode == "S" } {
	set mode3D(sticks) On
	xc_displayMode3D .mesa over S
    } elseif { $mode == "PB" } {
	set mode3D(pipe)   On
	set mode3D(sticks) On
	set mode3D(balls)  On
	xc_displayMode3D .mesa over PB
    } elseif { $mode == "BS" } {
	set mode3D(pipe)   Off
	set mode3D(sticks) On
	set mode3D(balls)  On
	xc_displayMode3D .mesa over BS
    } elseif { $mode == "SF" } {
	set mode3D(space) On
	xc_displayMode3D .mesa over SF
    } else {
	puts stderr "invalid value \"$mode\" submitted to proc DisplayOver3D, must be one of S, BS, SF"
	flush stderr
    }
    return
}


# ============================================================================
# istead of "xc_displayMode3D boolean" this proc should be used
#            ^^^^^^^^^^^^^^^^^^^^^^^^
proc DispBool3D {var} {
    global mode3D
    
    if { $var == "mode3D(sticks)" } {
	set mode3D(space) Off
	Invert3D mode3D(sticks)
	xc_displayMode3D .mesa boolean S
    } elseif { $var == "mode3D(balls)" } {
	set mode3D(space) Off
	Invert3D mode3D(balls)
	xc_displayMode3D .mesa boolean B
    } elseif { $var == "mode3D(space)" } {
	set mode3D(sticks) Off
	set mode3D(balls)  Off
	Invert3D mode3D(space)
	xc_displayMode3D .mesa boolean SF
    }
    return
}


# this proc is for DispBool3D
proc Invert3D {var} {
    upvar #0 $var value
    puts stdout "Invert3D:: var=$var; value=$value\n"
    flush stdout
    if { $value == "On" } {
	set $var Off
    } else {
	set $var On
    }
}
    


# this proc map from 2D to 3D and display in 3D
proc From2Dto3D {} {
    global mode2D mode3D dispmode
    
    # mapping
    # (WF & PL)   --> Sticks (B)
    # (BS1 & BS2) --> BallSticks (BS)

    if ![info exists dispmode(mode3D)] { set dispmode(mode3D) Over }

    if { $mode2D(WF) == "On" } {
	set mode3D(sticks) On 
	xc_displayMode3D .mesa over S
    } elseif { $mode2D(PL) == "On" || $mode2D(PB) == "On" } {
	set mode3D(pipe)   On
	set mode3D(sticks) On
	set mode3D(balls)  On
	xc_displayMode3D .mesa over PB
    } elseif { $mode2D(BS1) == "On" || $mode2D(BS2) == "On" } {
	set mode3D(pipe)   Off
	set mode3D(sticks) On
	set mode3D(balls)  On
	xc_displayMode3D .mesa over BS
    } else {
	set mode3D(sticks) Off
	set mode3D(balls)  Off
	set mode3D(space)  On
	xc_displayMode3D .mesa over SF
    }

    set mode2D(WF)  Off
    set mode2D(PL)  Off
    set mode2D(PB)  Off
    set mode2D(BS1) Off
    set mode2D(BS2) Off
    set mode2D(SF)  Off

    # enable menubuttons for shade-model & draw-style
    set dispmode(style)   3D
    xcGetState
    return
}
	

# this proc map from 3D to 2D and display in 2D
proc From3Dto2D {} {
    global mode2D mode3D dispmode
    # mapping:
    # (S || ~B) --> WF
    # (B) --> BS2
    # (SF) --> BS2
    
    # if dispmode(mode3D) do not exists, just return silenlty, because
    # we were not yet on 3D modes and thus going from 3D to 2D is absurd
    if ![info exists dispmode(mode3D)] { 
	puts stdout "3D NOT EXIST"
	return }
	
    if { $mode3D(sticks) == "On" && $mode3D(balls) == "Off" } { 
	set mode2D(WF)   On 
	xc_displayMode2D .mesa WF
    } elseif { $mode3D(balls) == "On" && $mode3D(pipe) =="Off" } {
	set mode2D(BS2)  On
	xc_displayMode2D .mesa BS2
    } elseif { $mode3D(balls) == "On" && $mode3D(pipe) == "On" } {
	set mode2D(PB)   On
	xc_displayMode2D .mesa PB
    } else {
	set mode2D(SF)   On
	xc_displayMode2D .mesa SF
    }
    set mode3D(pipe)   Off
    set mode3D(sticks) Off
    set mode3D(balls)  Off
    set mode3D(space)  Off

    # enable menubuttons for shade-model & draw-style
    set dispmode(style)   2D
    xcGetState
    return
}


# this proc deletes all child-widgets of parent
proc DeleteWid {parent} {
    
    set child [pack slaves $parent]
    puts stdout "CHILD = $child\n"
    foreach widget $child {
	puts stdout "WIDGET = $widget\n"
	destroy $widget
    }
}


# reset mode2D and mode3D arrays
proc ResetDispModes {} {
    global mode2D mode3D light

    set light Off
    set mode2D(WF)  Off
    set mode2D(PL)  On
    set mode2D(BS1) Off
    set mode2D(BS2) Off

    set mode3D(sticks) Off
    set mode3D(balls)  Off
    set mode3D(space)  Off
    if [info exists dispmode(mode3D)] { unset dispmode(mode3D) }
}


proc ConfigControlWid {} {
    global widsize

    set width  [expr [winfo width .ctrl] - $widsize(scrW)]
    set height [winfo height .ctrl]
    set h      [winfo reqheight .ctrl.c.f]
    .ctrl.c config -width $width -height $height \
	    -scrollregion "0 0 $width $h" 
}


proc DisplayDefaultMode {} {
    global light

    set light On
    #Lighting On
    #Mode3D Over
    #DisplayOver3D BS
    #DisplayOver3D BS
    DisplayOver3D SF
    DisplayOver3D SF
}