#############################################################################
# Author:                                                                   #
# ------                                                                    #
#  Anton Kokalj                                  Email: Tone.Kokalj@ijs.si  #
#  Department of Physical and Organic Chemistry  Phone: x 386 1 477 3523    #
#  Jozef Stefan Institute                          Fax: x 386 1 477 3811    #
#  Jamova 39, SI-1000 Ljubljana                                             #
#  SLOVENIA                                                                 #
#                                                                           #
# Source: $XCRYSDEN_TOPDIR/Tcl/FS_Main.tcl
# ------                                                                    #
# Copyright (c) 1996-2003 by Anton Kokalj                                   #
#############################################################################

#
# NOTE: XSF file and (band)XSF file must alreadu be read !!!
#
proc FS_GoFermi {{spin {}}} {
    global fs xcMisc system

    if { ! [info exists fs(counter)] } {
	set fs(counter) 0
    } else {
	incr fs(counter) 
    }

    set fs($spin,togl_w) [expr 800 * $xcMisc(resolution_ratio1)]
    set fs($spin,togl_h) $fs($spin,togl_w) 

    set t [toplevel .fs${spin}$fs(counter)]
    if { $spin != {} } {
	wm title $t "*** XCrySDen - Fermi Surface ($spin spin): $fs(titlefile)"
    } else {
	wm title $t "*** XCrySDen - Fermi Surface: $fs(titlefile)"
    }
    wm iconname $t "Fermi Surface"    
    wm iconbitmap . @$system(BMPDIR)/xcrysden.xbm

    if { $spin == "dn" } {
	wm geometry $t +0-0
    } else {
	wm geometry $t -0+10
    }
    
    set nb [NoteBook $t.nb]
    pack $nb -expand 1 -fill both
    set fs($spin,nb) $nb

    set _first_band -1
    for {set i 1} {$i <= $fs($spin,nbands)} {incr i} {
	if $fs($spin,$i,band_selected) {
	    if { $_first_band == -1 } {
		set _first_band $i
	    }
	    # initialize variables
	    FS_InitVar $i $spin

	    $nb insert $i band$i -text "Band #$i" \
		    -createcmd 	[list FS_RenderSurface $i $spin]
	    set f [$nb getframe band$i]
	    set fs($spin,$i,togl) [togl $f.$i \
		    -ident $f.$i \
		    -width $fs($spin,togl_w) \
		    -height $fs($spin,togl_h) \
		    -rgba           true  \
		    -redsize        1     \
		    -greensize      1     \
		    -bluesize       1     \
		    -double         true  \
		    -depth          true  \
		    -depthsize      1     \
		    -accum          false \
		    -accumredsize   1     \
		    -accumgreensize 1     \
		    -accumbluesize  1     \
		    -accumalphasize 1     \
		    -alpha          false \
		    -alphasize      1     \
		    -stencil        false \
		    -stencilsize    1     \
		    -auxbuffers     0     \
		    -overlay        false \
		    -stereo         false \
		    -time           1]

	    pack $f.$i
	    bind $fs($spin,$i,togl) <B1-Motion> [list \
		    $fs($spin,$i,togl) xc_B1motion %x %y]
	    bind $fs($spin,$i,togl) <B2-Motion> [list \
		    $fs($spin,$i,togl) xc_B2motion %x %y]
	    bind $fs($spin,$i,togl) <B1-ButtonRelease> [list \
		    $fs($spin,$i,togl) xc_Brelease B1]
	    bind $fs($spin,$i,togl) <B2-ButtonRelease> [list \
		    $fs($spin,$i,togl) xc_Brelease B2 ]
	    bind $fs($spin,$i,togl) <Button-3> [list \
		    FS_PopupMenu %W %X %Y $i $spin]

	    set fs($spin,$i,ident) [cry_surfreg $fs($spin,$i,togl)]	    
	    cry_dispfunc $fs($spin,$i,togl) fermisurface
	    #
	    # status label here
	    #
	    set ff [frame $f.f -relief ridge -bd 4]
	    pack $ff -side top -expand 1 -fill x -padx 0m -pady 0m 
	    set fs($spin,$i,status_f) $ff
	    
	    if { $spin != {} } {
		set l1  [label $ff.l1 -text "Spin: [string toupper $spin]" \
			-anchor w -relief sunken -bd 1]
		pack $l1 -side left -padx 1m -ipadx 1m -ipady 1m
	    }
	    set l2 [label $ff.l2 -text "FERMI Energy: $fs(Efermi)" \
		    -anchor w -relief sunken -bd 1]
	    set l3 [label $ff.l3 -text "Min Ene: $fs($spin,$i,minE)" \
		    -anchor w -relief sunken -bd 1]
	    set l4 [label $ff.l4 -text "Max Ene: $fs($spin,$i,maxE)" \
		    -anchor w -relief sunken -bd 1]		
	    set fff [frame $ff.f -relief sunken -bd 1]
	    pack $l2 $l3 $l4 \
		    -side left -padx 1m -ipadx 1m -ipady 1m
	    pack $fff \
		    -side right -fill x -padx 1m -ipadx 1m -ipady 1m
	    
	    set l5 [label $fff.l5 -text "Isolevel:"]
	    set e  [entry $fff.e \
		    -width 15 -textvariable fs($spin,$i,isolevel)]
	    pack $e $l5 -side right -padx 0m -pady 0m
	    bind $e <Return> [list FS_Config $i $spin]	
	}
    }
    if { $_first_band != -1 } {
	$nb raise band$_first_band
    }

    update
    set fs($spin,top_w) [winfo width $t]
    set fs($spin,top_h) [winfo height $t]
    set fs($spin,top_togl_dw) [expr $fs($spin,top_w) - $fs($spin,togl_w)]
    set fs($spin,top_togl_dh) [expr $fs($spin,top_h) - $fs($spin,togl_h)]

    bind $t <Configure> [list FS_ResizeWin %W %w %h $t $spin]
}

proc FS_PopupMenu {W x y i {spin {}}} {
    global fs

    set togl $fs($spin,$i,togl)

    if { $fs($spin,$i,celltype) == "para" } {
	set cell cell
    } else {
	set cell "first Brillouin zone"
    }

    if [winfo exists $W.menu] {
	destroy $W.menu
    }
    set m [menu $W.menu -tearoff 1]
    tk_popup $m $x $y
    #$m add command -label "PopUp Menu" -state disabled
    #$m add separator
    $m add command -label "Render Surface" \
	    -command [list FS_RenderSurface $i $spin]
    $m add separator
    # -fs checkbuttons ######################################################
    $m add checkbutton -label "Display $cell" \
	    -variable fs($spin,$i,displaycell) \
	    -command [list FS_fsConfig $i $spin] 

    $m add checkbutton -label "Crop Fermi Surface to first BZ" \
	    -variable fs($spin,$i,cropbz) \
	    -command [list FS_fsConfig $i $spin]
    if { $fs($spin,$i,celltype) == "para" } {
	$m entryconfig "Crop Fermi Surface to first BZ" -state disabled
    }
    $m add cascade -label "Cell type" -menu $m.celltype
    $m add cascade -label "Display $cell as ..." -menu $m.discell    
    $m add separator

    # CELLTYPE CASCADE
    menu $m.celltype -tearoff 0
    $m.celltype add radiobutton -label "first Brillouin zone" \
	    -variable fs($spin,$i,text_celltype) \
	    -command [list celltype:FS_fsConfig $i $spin]
    $m.celltype add radiobutton -label "reciprocal primitive cell" \
	    -variable fs($spin,$i,text_celltype) \
	    -command [list celltype:FS_fsConfig $i $spin]
    
    # DISPLAYCELL CASCADE
    menu $m.discell -tearoff 0
    $m.discell add radiobutton -label "solid" \
	    -variable fs($spin,$i,celldisplaytype) \
	    -command [list FS_fsConfig $i $spin]
    $m.discell add radiobutton -label "wire" \
	    -variable fs($spin,$i,celldisplaytype) \
	    -command [list FS_fsConfig $i $spin]
    $m.discell add radiobutton -label "rod" \
	    -variable fs($spin,$i,celldisplaytype) \
	    -command [list FS_fsConfig $i $spin]
    $m.discell add radiobutton -label "solidwire" \
	    -variable fs($spin,$i,celldisplaytype) \
	    -command [list FS_fsConfig $i $spin]
    $m.discell add radiobutton -label "solidrod" \
	    -variable fs($spin,$i,celldisplaytype) \
	    -command [list FS_fsConfig $i $spin]

    $m.discell entryconfig "rod"      -state disabled
    $m.discell entryconfig "solidrod" -state disabled
    #########################################################################

    $m add cascade -image colors      -menu $m.colors 
    menu $m.colors -tearoff 0
    ColorMenu $W $m.colors
    $m add cascade -label "File"      -menu $m.file
    set mfile [menu $m.file -tearoff 0]

    $mfile add command -label "Print " -command [list printTogl $togl]
    $mfile add command -label "Save Fermi Surface(s) in BXSF format" \
	-command FS_Save

    $m add cascade -label "Display"   -menu $m.dis 
    menu $m.dis -tearoff 1
    FS_DisplayMenu $m.dis $W $i $spin

    $m add cascade -label "Modify"    -menu $m.mod 
    menu $m.mod -tearoff 1
    #FS_ModifyMenu $m.dis $W $i $spin

    $m add cascade -label "Tools"     -menu $m.tools 
    menu $m.tools -tearoff 0
    #FS_ToolsMenu $m.dis $W $i $spin
    $m add separator
    $m add command -label "Exit" -command exit
}

proc FS_ResizeWin {W w h t {spin {}}} {
    global fs

    if { $t != $W } {
	set w [winfo width $t]
	set h [winfo height $t]
    }
    # update only if size of "toplevel ." has changed
    if { $w != $fs($spin,top_w) || $h != $fs($spin,top_h) } {
	set fs($spin,top_w) $w
	set fs($spin,top_h) $h
	set fs($spin,togl_w) [expr $w - $fs($spin,top_togl_dw)]
	set fs($spin,togl_h) [expr $h - $fs($spin,top_togl_dh)]
	for {set i 1} {$i <= $fs($spin,nbands)} {incr i} {
	    if $fs($spin,$i,band_selected) {
		$fs($spin,$i,togl) config \
			-width  $fs($spin,togl_w) \
			-height $fs($spin,togl_h)
	    }
	}
    }
}

proc FS_InitVar {i {spin {}}} {
    global fs

    
    #
    # set monocolor:
    # 
    set rainbow {
	{ 1.0 0.2 0.2}
	{ 1.0 1.0 0.2}
	{ 0.2 1.0 0.2}
	{ 0.2 1.0 1.0}
	{ 0.2 0.2 1.0}
	{ 1.0 0.2 1.0}
    }
    set im [expr $i - 6 * int( $i / 6)]

    set fs($spin,$i,celltype)           bz
    set fs($spin,$i,text_celltype)      "first Brillouin zone"
    set fs($spin,$i,cropbz)             1
    set fs($spin,$i,displaycell)        1
    set fs($spin,$i,celldisplaytype)    wire
    set fs($spin,$i,drawstyle)          solid
    set fs($spin,$i,transparent)        0
    set fs($spin,$i,shademodel)         smooth
    set fs($spin,$i,monocolor)          [lindex $rainbow $im]
    set fs($spin,$i,smoothsteps)        0
    set fs($spin,$i,smoothweight)       0.2

    set fs($spin,$i,old_celltype)        $fs($spin,$i,celltype)
    set fs($spin,$i,old_cropbz)          $fs($spin,$i,cropbz)
    set fs($spin,$i,old_displaycell)     $fs($spin,$i,displaycell)
    set fs($spin,$i,old_celldisplaytype) $fs($spin,$i,celldisplaytype)
    set fs($spin,$i,old_drawstyle)       $fs($spin,$i,drawstyle)
    set fs($spin,$i,old_transparent)     $fs($spin,$i,transparent)
    set fs($spin,$i,old_shademodel)      $fs($spin,$i,shademodel)
    set fs($spin,$i,old_monocolor)       $fs($spin,$i,monocolor)
    set fs($spin,$i,old_smoothsteps)     $fs($spin,$i,smoothsteps)
    set fs($spin,$i,old_smoothweight)    $fs($spin,$i,smoothweight)
}

proc FS_RenderSurface {i {spin {}}} {
    global fs

    if ![info exist fs($spin,$i,rendered)] {
	set fs($spin,$i,rendered) 1
	
	FS:cry_surf $i $spin
	# next lines are a hack-around a "display-bug" to force the display
	set w [lindex [$fs($spin,$i,togl) config -width] end]
	$fs($spin,$i,togl) config -width $w
	$fs($spin,$i,togl) render
	$fs($spin,$i,togl) swapbuffers
	update
    }
}

proc FS:cry_surf {i {spin {}}} {
    global fs

    # band index-identifiers strats from 0 not from 1 in ReadBandGrid !!!
    set iband [expr $i - 1]
    SetWatchCursor
    update

    xcDebug -stderr "cry_surf $fs($spin,$i,togl) \
	    -ident $fs($spin,$i,ident) \
	    -type  fermisurface \
	    -fs    [list \
	        -gridindex       $fs($spin,grid_index) \
	        -gridsubindex    $fs($spin,grid_subindex) \
	        -bandindex       $iband \
	        -celltype        $fs($spin,$i,celltype) \
	        -cropbz          $fs($spin,$i,cropbz)  \
	        -displaycell     $fs($spin,$i,displaycell) \
	        -celldisplaytype $fs($spin,$i,celldisplaytype)] \
	    -level        $fs($spin,$i,isolevel) \
	    -drawstyle    $fs($spin,$i,drawstyle) \
	    -transparent  $fs($spin,$i,transparent) \
	    -shademodel   $fs($spin,$i,shademodel) \
	    -monocolor    $fs($spin,$i,monocolor) \
	    -smoothsteps  $fs($spin,$i,smoothsteps) \
	    -smoothweight $fs($spin,$i,smoothweight)"


    cry_surf $fs($spin,$i,togl) \
	    -ident $fs($spin,$i,ident) \
	    -type  fermisurface \
	    -fs    [list \
	        -gridindex       $fs($spin,grid_index) \
	        -gridsubindex    $fs($spin,grid_subindex) \
	        -bandindex       $iband \
	        -celltype        $fs($spin,$i,celltype) \
	        -cropbz          $fs($spin,$i,cropbz)  \
	        -displaycell     $fs($spin,$i,displaycell) \
	        -celldisplaytype $fs($spin,$i,celldisplaytype)] \
	    -level        $fs($spin,$i,isolevel) \
	    -drawstyle    $fs($spin,$i,drawstyle) \
	    -transparent  $fs($spin,$i,transparent) \
	    -shademodel   $fs($spin,$i,shademodel) \
	    -monocolor    $fs($spin,$i,monocolor) \
	    -smoothsteps  $fs($spin,$i,smoothsteps) \
	    -smoothweight $fs($spin,$i,smoothweight)
    ResetCursor 
    update
}


proc FS:cry_surfconfig {i {spin {}}} {
    global fs

    if ![info exist fs($spin,$i,rendered)] {
	return
    }

    # band index-identifiers strats from 0 not from 1 in ReadBandGrid !!!
    set iband [expr $i - 1]

    SetWatchCursor
    update
    cry_surfconfig $fs($spin,$i,togl) \
	    -ident $fs($spin,$i,ident) \
	    -fs    [list \
	        -gridindex       $fs($spin,grid_index) \
	        -gridsubindex    $fs($spin,grid_subindex) \
	        -bandindex       $iband \
	        -celltype        $fs($spin,$i,celltype) \
	        -cropbz          $fs($spin,$i,cropbz)  \
	        -displaycell     $fs($spin,$i,displaycell) \
	        -celldisplaytype $fs($spin,$i,celldisplaytype)] \
	    -render       1 \
	    -level        $fs($spin,$i,isolevel) \
	    -drawstyle    $fs($spin,$i,drawstyle) \
	    -transparent  $fs($spin,$i,transparent) \
	    -shademodel   $fs($spin,$i,shademodel) \
	    -monocolor    $fs($spin,$i,monocolor) \
	    -smoothsteps  $fs($spin,$i,smoothsteps) \
	    -smoothweight $fs($spin,$i,smoothweight)
    ResetCursor 
    update
}


proc FS_DisplayMenu {m togl i {spin {}}} {
    global fs

    # band index-identifiers strats from 0 not from 1 in ReadBandGrid !!!
    $m add checkbutton -label "Transparent Surface" \
	    -variable fs($spin,$i,transparent) \
	    -command [list FS_Config $i $spin]

    $m add separator
    $m add cascade -label "Surface Drawstyle ..."  -menu $m.draw
    $m add cascade -label "Surface Shademodel ..." -menu $m.shade
    $m add command -label "Set Surface Color" \
	    -command [list FS_SetSurfColor $i $spin]
    $m add command -label "Surface Smoothing" \
	    -command [list FS_SurfSmooth $i $spin]

    # DRAWSTYLE
    menu $m.draw -tearoff 0
    $m.draw add radiobutton -label "solid" \
	    -variable fs($spin,$i,drawstyle) \
	    -command [list FS_Config $i $spin]
    $m.draw add radiobutton -label "wire" \
	    -variable fs($spin,$i,drawstyle) \
	    -command [list FS_Config $i $spin]
    $m.draw add radiobutton -label "dot" \
	    -variable fs($spin,$i,drawstyle) \
	    -command [list FS_Config $i $spin]

    # SHADEMODEL
    menu $m.shade -tearoff 0
    $m.shade add radiobutton -label "smooth" \
	    -variable fs($spin,$i,shademodel) \
	    -command [list FS_Config $i $spin]
    $m.shade add radiobutton -label "flat" \
	    -variable fs($spin,$i,shademodel) \
	    -command [list FS_Config $i $spin]
}


proc FS_fsConfig {i {spin {}}} {
    global fs

    if { $fs($spin,$i,old_celltype) != $fs($spin,$i,celltype) } {
	FS:cry_surf $i $spin
    } else {
	FS:cry_surfconfig $i $spin
    }
    set fs($spin,$i,old_celltype) $fs($spin,$i,celltype)
}

proc FS_Config {i {spin {}}} {
    global fs

    # -level        $fs($spin,$i,isolevel)
    # -drawstyle    $fs($spin,$i,drawstyle)
    # -transparent  $fs($spin,$i,transparent)
    # -shademodel   $fs($spin,$i,shademodel)
    # -monocolor    $fs($spin,$i,monocolor)
    # -smoothsteps  $fs($spin,$i,smoothsteps)
    # -smoothweight $fs($spin,$i,smoothweight)

    FS:cry_surfconfig $i $spin    
}
    

proc celltype:FS_fsConfig {i {spin {}}} {
    global fs

    if { $fs($spin,$i,text_celltype) == "reciprocal primitive cell" } {
	set fs($spin,$i,celltype) para
    } else {
	set fs($spin,$i,celltype) bz
    }

    FS_fsConfig $i $spin
}

    
proc FS_SetSurfColor {i {spin {}}} {
    global fs

    SurfaceChangeCol fs($spin,$i,monocolor)
    FS_Config $i $spin
}


proc FS_SurfSmooth {i {spin {}}} {
    global fs fs_trial

    set fs_trial($spin,$i,smoothsteps)   $fs($spin,$i,smoothsteps) 
    set fs_trial($spin,$i,smoothweight)	 $fs($spin,$i,smoothweight)

    set t [xcToplevel [WidgetName] "Surface Smoothing" "SurfSmooth" . 20 20 1]

    message $t.m -aspect 800 \
	    -relief groove -bd 2 \
	    -text "Reasonable values for weight are between 0.1 and 1. Lighter weight will require more steps for smoothing, but will perturb the surface less !!!"
    pack $t.m -side top -padx 3m -pady 3m -ipadx 1m -ipady 1m 

    set f [frame $t.f]	        
    set e [FillEntries $t {"Smoothing steps:" "Smoothing weight:"} \
	    [list fs_trial($spin,$i,smoothsteps) \
	    fs_trial($spin,$i,smoothweight)] 17 7]
    set e1 [string trimright $e 1]
    set foclist "$e $e1"
    set varlist [list \
	[list fs_trial($spin,$i,smoothsteps) int] \
	[list fs_trial($spin,$i,smoothweight) real] ]

    button $t.b1 -text "Close"  -command [list CancelProc $t]
    button $t.b2 -text "Update" \
	    -command [list FS_SurfSmoothOK $t $foclist $varlist $i $spin]
    
    pack $f -side bottom -expand 1 -fill both  -padx 3m -pady 3m
    pack $t.b1 $t.b2 -side left -expand 1 -padx 2m -pady 2m
}
proc FS_SurfSmoothOK {t foclist varlist i {spin {}}} {
    global fs fs_trial
    
    if ![check_var $varlist $foclist] {
	return
    }
    set fs($spin,$i,smoothsteps)   $fs_trial($spin,$i,smoothsteps) 
    set fs($spin,$i,smoothweight)  $fs_trial($spin,$i,smoothweight)
    FS_Config $i $spin

    return
}


proc SurfaceChangeCol varname {
    upvar #0 $varname var

    set c1 [d2h [expr [lindex $var 0] * 255]]
    set c2 [d2h [expr [lindex $var 1] * 255]]
    set c3 [d2h [expr [lindex $var 2] * 255]]

    set t [xcToplevel [WidgetName] "Set Canvas Background" "SetBg" . 0 0 1]
    xcModifyColor $t "Set Surface Monocolor" #${c1}${c2}${c3} groove \
	    left left 100 100 70 5 20

    proc SurfaceChangeColOK {type t varname} {
	upvar #0 $varname var
	global mody_col mody
	
	if { $type == "OK" } {
	    set cID [xcModifyColorGetID]
	    set var [list $mody_col($cID,red) \
		    $mody_col($cID,green) $mody_col($cID,blue) 1.0]
	}
	destroy $t
    }

    set ok  [DefaultButton [WidgetName $t] -text "OK" \
	    -command [list SurfaceChangeColOK OK     $t $varname]]
    set can [button [WidgetName $t] -text "Cancel" \
	    -command [list SurfaceChangeColOK Cancel $t $varname]]
    pack $ok $can -padx 10 -pady 10 -expand 1
    tkwait window $t
}
