#############################################################################
# Author:                                                                   #
# ------                                                                    #
#  Anton Kokalj                                  Email: Tone.Kokalj@ijs.si  #
#  Department of Physical and Organic Chemistry  Phone: x 386 1 477 3523    #
#  Jozef Stefan Institute                          Fax: x 386 1 477 3811    #
#  Jamova 39, SI-1000 Ljubljana                                             #
#  SLOVENIA                                                                 #
#                                                                           #
# Source: $XCRYSDEN_TOPDIR/Tcl/gifAnim.tcl
# ------                                                                    #
# Copyright (c) 1996-2003 by Anton Kokalj                                   #
#############################################################################

proc gifAnimWidPack {frame can} {
    global gifAnim

    set gifAnim(X) [winfo width  $can]
    set gifAnim(Y) [winfo height $can]
    if { $gifAnim(button_text) == "Animated GIF/MPEG >>" } {
	pack $frame -side top 
	set gifAnim(button_text) "<< Animated GIF/MPEG"
    } else {
	pack forget $frame
	set gifAnim(button_text) "Animated GIF/MPEG >>"
    }
}


proc gifAnimMake {can} {
    global gifAnim xcMisc mesa_bg system
    
    if { $gifAnim(make_text) == "Start Recording Animated-GIF/MPEG" } {
	set gifAnim(make_text) "Stop Recording and Save"
	set gifAnim(make_gifAnim) 1
	set gifAnim(filelist)     {}  
	set gifAnim(nframe)       0
    } else {
	set gifAnim(make_text) "Start Recording Animated-GIF/MPEG"
	set gifAnim(make_gifAnim) 0
	
	#
	# now create the Animated-GIF/MPEG
	#
	if { $gifAnim(mpeg) == 1 } {
	    set separator \n
	} else {
	    set separator " "
	}
	set first_frame [lindex $gifAnim(filelist) 0]
	set last_frame  [lindex $gifAnim(filelist) end]
	set first {}
	set last  {}
	if { $gifAnim(ntime_first_frame) > 1 } {
	    for {set i 1} {$i < $gifAnim(ntime_first_frame)} {incr i} {
		append first "${first_frame}$separator"
	    }
	}
	if { $gifAnim(ntime_last_frame) > 1 } {
	    for {set i 1} {$i < $gifAnim(ntime_last_frame)} {incr i} {
		append last "${last_frame}$separator"
	    }
	}
	set filelist [format "%s${separator}%s${separator}%s" $first $gifAnim(filelist) $last]

	if { $gifAnim(mpeg) == 0 } {
	    #
	    # create Animated GIF
	    set outfile $system(PWD)/anim.gif	    
	    
	    if { $gifAnim(transp) } {
		if { [string index $mesa_bg(current) 0] == "#" } {
		    set bg_color $mesa_bg(current)
		} else {
		    set bg_color [rgb_f2h $mesa_bg(current)]
		}
		# this doesn't work yet in WHIRLGIF
		#append flags " -transp $bg_color"
	    }
	    if { $gifAnim(global_colormap) } {
		append flags " -globalmap"
	    }
	    set args [format "%s %s %s %s %s" $xcMisc(whirlgif) $flags -o $outfile $filelist]
	    eval xcCatchExecRedirectStdErr $args
	} else {
	    #
	    # create MPEG
	    set outfile $system(PWD)/anim.mpg
	    
	    set pfile [mpegCreateParamFile $outfile $system(SCRDIR) $filelist]
	    xcCatchExecRedirectStdErr $xcMisc(mpeg_encode) $pfile
	}
    }
}

	
proc gifAnimPrintCurrent {can} {
    global gifAnim xcMisc mesa_bg system

    # first determine some variables
    set head anim-$gifAnim(nframe)
    if { $gifAnim(mpeg) == 0 } {
	set file $head.gif
	append gifAnim(filelist) " $file"
    } else {
	set file $head.ppm
	#t.k
	#append gifAnim(filelist) "$file\n"
	append gifAnim(filelist) "$head.jpg\n"
    }

    incr gifAnim(nframe)

    # determine the right geometry
    set canX [winfo width  $can]
    set canY [winfo height $can] 

    set canAspect [expr double($canX) / double($canY)]
    set gifAspect [expr double($gifAnim(X)) / double($gifAnim(Y))]
		   
    set dX [expr $gifAnim(X) - $canX]
    set dY [expr $gifAnim(Y) - $canY]

    if { $canAspect > $gifAspect } {
	# align canX --> gifX
	if { $dX > 0 } {
	    set geomX   $gifAnim(X)
	} else {
	    set geomX   $canX
	}	
	set geomY   [expr round(double($geomX) / $canAspect)]
	set borderX 0
	set borderY [expr round(0.5 * ($gifAnim(Y) - $geomY))]
    } else {
	# align canY --> gifY
	if { $dY > 0 } {
	    set geomY $gifAnim(Y)
	} else {
	    set geomY $canY
	}
	set geomX   [expr round(double($geomY) * $canAspect)]
	set borderX [expr round(0.5 * ($gifAnim(X) - $geomX))]
	set borderY 0
    }
    if { ![info exists mesa_bg(current)] } {
	set mesa_bg(current) {0.0 0.0 0.0 1.0}
    }
    if { [string index $mesa_bg(current) 0] == "#" } {
	set bg_color $mesa_bg(current)
    } else {
	set bg_color [rgb_f2h $mesa_bg(current)]
    }

    # now update the display (maybe $can was obscured)
    SetWatchCursor
    $can cry_dump2ppm $system(SCRDIR)/tmp.ppm 1
    
    set ext [file extension [file tail $file]]
    if { [string toupper $ext] != ".PPM" } {
	exec $xcMisc(ImageMagick.convert) $system(SCRDIR)/tmp.ppm \
	    $system(SCRDIR)/$file 
    } else {
	# t.k
	#exec cp $system(SCRDIR)/tmp.ppm $system(SCRDIR)/$file
	exec $xcMisc(ImageMagick.convert) $system(SCRDIR)/tmp.ppm \
	    $system(SCRDIR)/$head.jpg 	
    }
    
    ResetCursor    
    
    #
    # first dump with xwd and then conert to wanted format
    #
    # after idle [list xcCatchExecRedirectStdErr \
    # 		    xwd -id [winfo id $can] | $xcMisc(ImageMagick.convert) \
    # 		    -antialias -geometry ${geomX}x${geomY} \
    # 		    -bordercolor $bg_color \
    # 		    -border ${borderX}x${borderY} \
    # 		    - $system(SCRDIR)/$file]
    
    # after idle [list xcCatchExec xwd -id [winfo id $can] \
    # 		    -out $system(SCRDIR)/$head.xwd]
    # update idletasks
    # eval xcCatchExec $xcMisc(ImageMagick.convert) $flags \
    # 	$system(SCRDIR)/$head.xwd $system(SCRDIR)/$head.gif
    # ResetCursor    
}