#############################################################################
# Author:                                                                   #
# ------                                                                    #
#  Anton Kokalj                                  Email: Tone.Kokalj@ijs.si  #
#  Department of Physical and Organic Chemistry  Phone: x 386 1 477 3523    #
#  Jozef Stefan Institute                          Fax: x 386 1 477 3811    #
#  Jamova 39, SI-1000 Ljubljana                                             #
#  SLOVENIA                                                                 #
#                                                                           #
# Source: $XCRYSDEN_TOPDIR/Tcl/loadAttributes.tcl
# ------                                                                    #
# Copyright (c) 1996-2003 by Anton Kokalj                                   #
#############################################################################

proc xsfLoadAttributes {file} {
    global mody
    
    set elementcolor  0
    set elementradius 0
    set genericattr   0         
    foreach line [split [ReadFile $file] \n] {
	xcDebug -stderr "---> $elementradius ;;; $line"
	if { [llength $line] == 0 } {
	    # an empty-line --> skip it
	    continue
	} elseif { [regexp -- {^[a-zA-Z]a*} $line] } {
	    set keyword [string trim $line { }]; # get rid of trailing white-spaces
	    switch -glob -- $keyword {
		"ELEMENTCOL*" { 
		    set elementcolor  1 
		    set elementradius 0
		    set genericattr   0         
		}
		"ELEMENTRAD*" {
		    set elementcolor  0
		    set elementradius 1
		    set genericattr   0         
		}
		default {
		    set elementcolor  0
		    set elementradius 0
		    set genericattr   1         

		    set key     [lindex $keyword 0]
		    set values  [lrange $keyword 1 end]
		    xcDebug -stderr "----> generic option: $keyword $values"
		    if { ! [info exists mody($key)] } {
			ErrorDialog "while parsing attribute file \"$file\", while reading string \"$keyword\""
		    }
		    if { [catch {eval {xc_newvalue .mesa $mody($key)} $values} error ] } {
			ErrorDialog "while parsing attribute file \"$file\", while reading string \"$keyword\".\n\nError message: $error"
		    }
		}	
	    }
	} else {
	    if { $elementcolor } {

		if { [llength $line] != 4 } {
		    ErrorDialog "should get 4 numbers (i.e. atomic-number r g b), but got: $line\nwhile reading attribute file: $file"
		    return
		}
		xc_newvalue .mesa $mody(L_ATCOL_ONE) [lindex $line 0] \
		    [lindex $line 1]  [lindex $line 2]  [lindex $line 3]

	    } elseif { $elementradius } {

		if { [llength $line] != 2 } {
		    ErrorDialog "should get 2 numbers (i.e. atomic-number atomic-radius), but got: $line\nwhile reading attribute file: $file"
		    return
		}
		xc_newvalue .mesa $mody(L_ATRAD_ONE) [lindex $line 0] \
		    [lindex $line 1]  
	    }
	}
    }
}
		