#############################################################################
# Author:                                                                   #
# ------                                                                    #
#  Anton Kokalj                                  Email: Tone.Kokalj@ijs.si  #
#  Department of Physical and Organic Chemistry  Phone: x 386 1 477 3523    #
#  Jozef Stefan Institute                          Fax: x 386 1 477 3811    #
#  Jamova 39, SI-1000 Ljubljana                                             #
#  SLOVENIA                                                                 #
#                                                                           #
# Source: $XCRYSDEN_TOPDIR/Tcl/parseComLinArg.tcl                               #
# ------                                                                    #
# Copyright (c) 1996-2003 by Anton Kokalj                                   #
#############################################################################

proc UpdateWMTitle file {
    global xcMisc system
    
    set xcMisc(titlefile) $file
    wm title . "XCrySDen: [file tail $file]"
    # the following copy is used if someone wants to save 
    # file as WIEN97 STRUCT FILE
    exec cp $file $system(SCRDIR)/xc_struc.$system(PID)
}


proc XCrySDenUsage {} {
    global system

    wm withdraw .
    set message [ReadFile $system(TOPDIR)/usage]
    puts stderr "\n$message\n"
    addOption:printCustomUsage
    flush stderr
    exit 1
}


proc parseComLinArg comopt {
    global system

    set load_attributes 0
    set i 0
    xcDebug "ARGS = $comopt"
    foreach option $comopt {
	incr i
	xcDebug "option=$option; i=$i"
	# odd cycles are tags, even options
        if { $i%2 } {
            set tag $option
	    switch -- $tag {
		"-?"   { 
		    XCrySDenUsage
		}
		"--help" {
		    XCrySDenUsage
		}
		"--quiet" {
		    # quiet run -- withput welcome window !!!
		    ViewMol .
		    return
		}
	    }
	} else {	    
	    set filedir [GetAbsoluteFileName $option]
            switch -glob -- $tag {
		"-a" -
		"--attrib*" {
		    set load_attributes 1
		    set file_attributes $filedir
		}
                "--xsf" -
		"--xcr" {
		    ViewMol .
		    xsfOpen $filedir .mesa
		}
		"--animxsf" -
		"--axsf" {
		    ViewMol .
		    xsfAnimInit $filedir .mesa
		}
		"--xyz" {
		    # first convert to xsf format then set state and render
		    ViewMol .
		    OpenXYZPDB .mesa xyz $filedir
		}
		"--pdb" {
		    # first convert to xsf format then set state and render
		    ViewMol .
		    OpenXYZPDB .mesa pdb $filedir
		}
		"--fermi" -
		"--bxsf" {
		    if [file isfile $filedir] {
			genFSInit $filedir
		    } else {
			tk_dialog [WidgetName] ERROR \
				"ERROR: file $filedir doesn't exists\n(or is not a regular file) !!!" error 0 OK
			exit
		    }
		}

		"--pw_inp" - "--pwi" {
		    if [file exists $filedir] {
			# pop-up Viewer
			ViewMol .
			openExtStruct 3 crystal external \
				$system(TOPDIR)/scripts/pwi2xsf.sh \
				pwi2xsf.xsf_out \
				{PWSCF Input File} ANGS \
				-file $filedir \
				-preset pwInputPreset
		    } else {
			tk_dialog [WidgetName] ERROR \
				"ERROR: $filedir doesn't exists !!!" error 0 OK
			exit
		    }
		}

		"--pw_out" - "--pwo" {
		    if [file exists $filedir] {
			# pop-up Viewer
			ViewMol .
			openExtStruct 3 crystal external \
				$system(TOPDIR)/scripts/dummy.sh \
				pwo2xsf.xsf_out \
				{PWSCF Output File} ANGS \
				-file $filedir \
				-preset pwOutputPreset
		    } else {
			tk_dialog [WidgetName] ERROR \
				"ERROR: $filedir doesn't exists !!!" error 0 OK
			exit
		    }
		}

		    
		"--fhi_inpini" {
		    if [file exists $filedir] {
			# pop-up Viewer
			ViewMol .
			openExtStruct 3 crystal external34 \
				$system(BINDIR)/fhi_inpini2ftn34 \
				$system(ftn_name).34 {FHI98MD "ini.inp"} \
				BOHR -file $filedir \
				-preset [list fhiPreset inpini]
		    } else {
			tk_dialog [WidgetName] ERROR \
				"ERROR: $filedir doesn't exists !!!" error 0 OK
			exit
		    }
		}
		"--fhi_coord" {
		    if [file exists $filedir] {
			# pop-up Viewer
			ViewMol .
			openExtStruct 3 crystal external \
				$system(BINDIR)/fhi_coord2xcr \
				fhi_coord.xcr {FHI98MD "coord.out"} \
				BOHR -file $filedir \
				-preset [list fhiPreset coord]
		    } else {
			tk_dialog [WidgetName] ERROR \
				"ERROR: $filedir doesn't exists !!!" error 0 OK
			exit
		    }
		}
		# --crystal_* options are intended as CRYSTAL95/98 interfacing
		"--crystal_inp" {
		    if [file exists $filedir] {
			ViewMol .
			OpenFile $filedir
		    } else {
			tk_dialog [WidgetName] ERROR \
				"ERROR: $filedir file does not exists" error 0 OK
			exit
		    }
		}
		"--crystal_f9" {
		    if [file exists $filedir] {
			ViewMol .
			PropC95 $filedir
		    } else {
			tk_dialog [WidgetName] ERROR \
				"ERROR: $filedir file does not exists" error 0 OK
			exit
		    }
		}
		# --wien_* options are intended as WIEN97 interfacing
		"--wien_struct" {
		    # option can be directory, filehead or file
		    if [file isdirectory $filedir] {
			set filehead [file tail $filedir]
			set file $filedir/$filehead.struct
		    } elseif [file exists $filedir.struct] {
			set file $filedir.struct
		    } elseif [file exists $filedir] {
			set file $filedir
		    } else {
			tk_dialog [WidgetName] ERROR \
				"ERROR: $filedir is neither directory, neither filehead, niether file" error 0 OK
			exit
		    }
		    # pop-up Viewer
		    ViewMol .
		    wnOpenSFile $file; # maybe vice-versa first wnOpenSFile and then ViewMol file
		}
		"--wien_kpath" {
		    # struct file must be read, and converted
		    # pop-up just BZ Window
		    if [file isdirectory $filedir] {
			wnDetComOpt $filedir
			wnKPath $filedir
		    } else {
			tk_dialog [WidgetName] ERROR \
				"ERROR: $filedir directory does not exists !" \
				error 0 OK
			exit
		    }
		}
		"--wien_renderdensity" {
		    # use wnDensity proc and pop-up Viewer and 2D isoControl
		    # toplevel
		    if [file isdirectory $filedir] {
			set filehead [file tail $filedir]
			set file $filedir/$filehead.struct
			ViewMol .
			wnOpenSFile $file
			wnDensity $filedir
		    } else {
			tk_dialog [WidgetName] ERROR \
				"ERROR: $filedir directory does not exists !" \
				error 0 OK
			exit
		    }
		}
		"--wien_density" {
		    # popup Viewer & another toplevel for inquiring 2D or 3D
		    # plots; then grid window is displayed
		    if [file isdirectory $filedir] {
			set filehead [file tail $filedir]
			set file $filedir/$filehead.struct
			ViewMol .
			wnOpenSFile $file
			wnDetComOpt $filedir
			# pop-up decision window
			wnDensity2D_or_3D $filedir
		    } else {
			tk_dialog [WidgetName] ERROR \
				"ERROR: $filedir directory does not exists !" \
				error 0 OK
			exit
		    }		    
		}
		"--wien_f*" {
		    # struct file must be read, and converted
		    if [file isdirectory $filedir] {
			wnDetComOpt $filedir
			wnFSInit $filedir
		    } else {
			tk_dialog [WidgetName] ERROR \
				"ERROR: $filedir directory does not exists !" \
				error 0 OK
			exit
		    }
		}
		"--g98_out" {
		    # this option is done via new addOption interface ...
		    addOption:hardcoded \
			$system(TOPDIR)/scripts/g98toxsf.sh $filedir
		}
		"--script" {
		    scripting:source $filedir
		}
		default {
		    #
		    # check for user-custom options
		    addOption:parse $tag $filedir
		}
	    }
	}
    }
	
    if { $i%2 } {
	XCrySDenUsage
	#tk_dialog .mb_error1 Error \
	#	"ERROR: You called XCrySDen with an odd number of args !" \
	#	error 0 OK
    }
    unset i

    if { $load_attributes && [winfo exists .mesa] } {
	xsfLoadAttributes $file_attributes
    }
}

