#############################################################################
# Author:                                                                   #
# ------                                                                    #
#  Anton Kokalj                                  Email: Tone.Kokalj@ijs.si  #
#  Department of Physical and Organic Chemistry  Phone: x 386 1 477 3523    #
#  Jozef Stefan Institute                          Fax: x 386 1 477 3811    #
#  Jamova 39, SI-1000 Ljubljana                                             #
#  SLOVENIA                                                                 #
#                                                                           #
# Source: $XCRYSDEN_TOPDIR/Tcl/print.tcl
# ------                                                                    #
# Copyright (c) 1996-2003 by Anton Kokalj                                   #
#############################################################################

proc printTogl {togl} {
    global xcMisc system

    set head [file rootname [file tail $xcMisc(titlefile)]]

    set defext  .eps
    #
    # check if we have convert program
    #
    if { [info exists xcMisc(ImageMagick.convert)] } {
	set deffile $head.jpg
	set defext  .jpg
	set filetypes {
	    {{JPEG}       {.jpg} }
	    {{PPM}        {.ppm} }
	    {{PGM}        {.pgm} }
	    {{EPS}        {.eps} }
	    {{PNG}        {.png} }
	    {{GIF}        {.gif} }
	    {{TIFF}       {.tiff}}
	    {{All Files}  *      }
	}
    } else {	
	set filetypes {
	    {{EPS}        {.eps} }
	    {{PPM}        {.ppm} }
	    {{PGM}        {.pgm} }
	    {{All Files}  *      }
	}
	set deffile $head.eps
    }
    
    set sfile [tk_getSaveFile -initialdir $system(PWD) \
		   -title "Print to File" \
		   -defaultextension $defext \
		   -initialfile $deffile \
		   -filetypes $filetypes]
    if { $sfile == {} } {
	return 0
    }
    
    
    set ext [file extension [file tail $sfile]]
    
    SetWatchCursor
    if { [string toupper $ext] == ".EPS" || \
	     [string toupper $ext] == ".PS" } {  
	printToglEPS $togl $sfile
    } elseif { [string toupper $ext] == ".PPM" } {
	$togl cry_dump2ppm $sfile 1 1 1
    } elseif { [string toupper $ext] == ".PGM" } {
	$togl cry_dump2ppm $sfile 2
    } else {
	$togl cry_dump2ppm $system(SCRDIR)/tmp.ppm 1
	exec $xcMisc(ImageMagick.convert) $system(SCRDIR)/tmp.ppm $sfile 
    }
    
    ResetCursor    
}


proc printWidget {widget} {
    global xcMisc system

    set head [file rootname [file tail $xcMisc(titlefile)]]

    set defext  .eps
    #
    # check if we have convert program
    #
    if { [info exists xcMisc(ImageMagick.convert)] } {
	tk_messageBox -message "WARNING:\n\nYou are about to dump the content of a widget.Please make sure that no window obscures the widget to be printed. This is because XCRYSDEN's print is an XWD window dump." -type ok -icon warning


	set deffile $head.jpg
	set defext  .jpg
	set filetypes {
	    {{JPEG}       {.jpg} }
	    {{EPS}        {.eps} }
	    {{PNG}        {.png} }
	    {{GIF}        {.gif} }
	    {{TIFF}       {.tiff}}
	    {{All Files}  *      }
	}
    } else {	
	tk_messageBox -message "WARNING:\n\nIn order to dump an xcMisc(ImageMagick.convert) variables must be defined. Please define it in the \$HOME/.xcrysden/custom-definitions file. The variable should hold the path of the XWD-to-xxx (xxx=PNG,GIF,JPG,...) format. Example:\n\n
set xcMisc(ImageMagick.convert) /usr/bin/convert" -type ok -icon warning

	return
    }

    set sfile [tk_getSaveFile -initialdir $system(PWD) \
		   -title "Print to File" \
		   -defaultextension $defext \
		   -initialfile $deffile \
		   -filetypes $filetypes]
    if { $sfile == {} } {
	return 0
    }
    
    set ext [file extension [file tail $sfile]]

    SetWatchCursor

    #
    # first dump with xwd and then conert to wanted format
    #
    set head [file rootname [file tail $xcMisc(titlefile)]]
    raise $widget
    update
    after 1000
    set id [winfo id $widget]
    puts stderr "PRINTING WINDOW: $id [winfo id .mesa]"
    after idle [list xcCatchExec xwd -id $id -out $system(SCRDIR)/$head.xwd]
    update idletasks
    # convert -gaussian 2x2 -geometry 200%
    # THIS SHOULD be catched !!!
    xcCatchExec $xcMisc(ImageMagick.convert) $system(SCRDIR)/$head.xwd $sfile
    ResetCursor    
}


proc printToglEPS {togl file} {
    global light toglEPS

    set t [xcToplevel [WidgetName] "EPS Printing Options" "Print" . 100 100 1]

    if { ! [info exists toglEPS(format)] } {
	set toglEPS(format)    EPS
    }
    if { ! [info exists toglEPS(pointsize)] } {
	set toglEPS(pointsize) 2.0
    }
    if { ! [info exists toglEPS(linewidth)] } {
	set toglEPS(linewidth) 2.0
    }

    if { $light == "On" } {
	#
	# ligting-ON mode
	#
	set toglEPS(sort)    BSP_SORT
	set toglEPS(EPStype) BITMAP
    } else {
	#
	# ligting-OFF mode
	#
	set toglEPS(sort)    NO_SORT
	set toglEPS(EPStype) VECTORIAL
    }
    
    
    set f1 [frame $t.1]
    set f2 [frame $t.2]
    set f0 [frame $t.0]
    set f3 [frame $t.3]
    pack $f1 $f2 -side top -expand 1 -fill x
    pack $f0 -fill both
    pack $f3 -side bottom -expand 0 -fill x

    set toglEPS(frame2) $f2

    set m  [xcMenuButton $f1 \
		-labeltext    "Type of PostScript:" \
		-labelwidth   19 \
		-textvariable toglEPS(EPStype) \
		-menu {
		    BITMAP    {set toglEPS(EPStype) BITMAP}
		    VECTORIAL {set toglEPS(EPStype) VECTORIAL}
		}]
    pack $m -side top -padx 5 -expand 1 -fill x

    trace variable toglEPS(EPStype) w xcTrace


    set f21 [frame $f2.1]
    set f22 [frame $f2.2]
    set f23 [frame $f2.3]
    set f24 [frame $f2.4]
    pack $f21 $f22 $f23 $f24 -side top -padx 5 -expand 1 -fill x

    # format sort options pointsize linewidth

    set m1 [xcMenuButton $f21 \
		-labeltext "Output format:" \
		-labelwidth 19 \
		-textvariable toglEPS(format) \
		-menu {
		    PS  {set toglEPS(format) PS}
		    EPS {set toglEPS(format) EPS}
		    TEX {set toglEPS(format) TEX}
		}]
    set m2 [xcMenuButton $f22 \
		-labeltext  "Sorting:" \
		-labelwidth 19 \
		-textvariable toglEPS(sort) \
		-menu {
		    NO_SORT     {set toglEPS(sort) NO_SORT}
		    SIMPLE_SORT {set toglEPS(sort) SIMPLE_SORT}
		    BSP_SORT    {set toglEPS(sort) BSP_SORT}
		}]

    pack $m1 $m2 -side top -padx 5 -pady 5 -expand 1 -fill x

    Entries $f23 {"Point size:" "Line width:"} {
	toglEPS(pointsize) toglEPS(linewidth)} 5

    label $f23.__l -text "Don't touch below settings,\nunless you know what you are doing !!!" -relief ridge -bd 2
    pack $f23.__l -side top -padx 5 -pady 5 -ipadx 5 -ipady 5 -fill x -expand 0

    label $f24.__l -text "GL2PS Options:"
    pack  $f24.__l -side left

    CheckVarButtons $f24 {
	DRAW_BACKGROUND    
	SIMPLE_LINE_OFFSET 
	SILENT             
	BEST_ROOT          
	OCCLUSION_CULL     
	NO_TEXT            
	LANDSCAPE          
	NO_PS3_SHADING     
    } {
	toglEPS(DRAW_BACKGROUND)
	toglEPS(SIMPLE_LINE_OFFSET) 
	toglEPS(SILENT)             
	toglEPS(BEST_ROOT)          
	toglEPS(OCCLUSION_CULL)     
	toglEPS(NO_TEXT)            
	toglEPS(LANDSCAPE)          
	toglEPS(NO_PS3_SHADING)     
    } top

    # bottom buttons
    set ok  [button $f3.print -text "Print" -default active \
		-command [list printToglEPS_Print $t $togl $file]]
    set can [button $f3.cancel -text "Cancel" \
		 -command [list CancelProc $t]]
    pack $ok $can -side right -padx 5 -pady 10 -expand 1
}


proc printToglEPS_Print {t togl file} {
    global toglEPS light

    if { $toglEPS(EPStype) == "BITMAP" } {
	destroy $t
	SetWatchCursor
	set colorEPS 1
	$togl xc_dump2eps $file $colorEPS    	
	ResetCursor
    } else {
	if { $light == "On" } {
	    set ans [tk_messageBox -icon info -message "NOTICE: Printing vectorial PostScript in \"Ligting-On mode\" may take long time, and the PostScript file size can reach up to several 100 MB.\n\nWould you like to continue?" -type yesno]
	    if { $ans == "no" } {		
		return
	    }
	}
	
	destroy $t
	
	set _options GL2PS_NONE
	foreach elem {
	    DRAW_BACKGROUND
	    SIMPLE_LINE_OFFSET 
	    SILENT             
	    BEST_ROOT          
	    OCCLUSION_CULL     
	    NO_TEXT            
	    LANDSCAPE          
	    NO_PS3_SHADING     
	} {
	    if { $toglEPS($elem) == 1 } {
		lappend _options GL2PS_$elem
	    }
	}

	xcDebug -stderr "Executing: $togl cry_gl2psPrintTogl \
	    GL2PS_$toglEPS(format) \
	    GL2PS_$toglEPS(sort) \
	    $_options $toglEPS(pointsize) $toglEPS(linewidth) $file"

	SetWatchCursor
	$togl cry_gl2psPrintTogl \
	    GL2PS_$toglEPS(format) \
	    GL2PS_$toglEPS(sort) \
	    $_options $toglEPS(pointsize) $toglEPS(linewidth) $file
	ResetCursor
    }
}