#############################################################################
# Author:                                                                   #
# ------                                                                    #
#  Anton Kokalj                                  Email: Tone.Kokalj@ijs.si  #
#  Department of Physical and Organic Chemistry  Phone: x 386 1 477 3523    #
#  Jozef Stefan Institute                          Fax: x 386 1 477 3811    #
#  Jamova 39, SI-1000 Ljubljana                                             #
#  SLOVENIA                                                                 #
#                                                                           #
# Source: $XCRYSDEN_TOPDIR/Tcl/selection.tcl                                       
# ------                                                                    #
# Copyright (c) 1996-2003 by Anton Kokalj                                   #
#############################################################################

#
# warning:: some outer procs use select global variable as well, so 
# You should never unset the selct variabl after done/selected/cancel 
# button was closed
#
proc SelCheck {com oper} {
    global maxsel nsel
    #com .... command to exexute if OK
    #oper ... name of command/operation to execute if OK
    
    if { $maxsel != $nsel } {
	dialog .selcheck Warning "WARNING: You must select $maxsel \
		atoms for $oper" warning 0 OK
	return
    } else {
	eval $com
    }
}


proc Done {w com {tplw {}}} {
    global maxsel nsel select text SelPlane SelAtomCL SelHoleCL SelLabel \
	    SelLine
    
    # w...name of canvas
    # tlpw...name of selection toplevel
    puts stdout "W,COM>$w; $com"
    if { $nsel == $maxsel } {
	for {set i 1} {$i <= $maxsel} {incr i} {
	    set select(Nat$i) [lindex $select(obj,$i) 2]
	    set select(X$i) [lindex $select(obj,$i) 3]
	    set select(Y$i) [lindex $select(obj,$i) 4]
	    set select(Z$i) [lindex $select(obj,$i) 5]
	}
	# this 'ifs' are for PLANE SELECTION -- AdvOption
	if { [lindex $com 0] != "PlaneSel" && \
		[lindex $com 0] != "PlaneSelSel" && \
		[lindex $com 0] != "SelCentreAtomCLSel" && \
		[lindex $com 0] != "LabelSelSel" && \
		[lindex $com 0] != "LineSelSel" && \
		[lindex $com 0] != "ParalleSel" && \
		[lindex $com 0] != "AtomSel" } {	    
	    puts stdout "COM> $com"
	    eval $com
	    return
	} elseif { [lindex $com 0] == "PlaneSel" } {
	    Plane
	    return
	} elseif { [lindex $com 0] == "SelCentreAtomCL" } {
	    SelCentreAtomCL
	    return
	} elseif { [lindex $com 0] == "PlaneSelSel" } {
	    set SelPlane(plane) [Plane]
	    puts stdout "DONE,PLANE> $SelPlane(plane)"
	    DeSel $tplw $w
	    set SelPlane(done) 1
	    return $SelPlane(plane)
	} elseif { [lindex $com 0] == "SelCentreAtomCLSel" } {
	    set SelAtomCL(centre) [SelCentreAtomCL]
	    DeSel $tplw $w
	    set SelAtomCL(done) 1
	    return $SelAtomCL(centre)
	} elseif { [lindex $com 0] == "LabelSelSel" } {
	    set SelLabel(label) [LabelSel]
	    set SelLabel(done) 1
	    DeSel $tplw $w
	    return $SelLabel(label)
	} elseif { [lindex $com 0] == "LineSelSel" } {
	    set SelLine(coor) [LineSel]
	    set SelLine(done) 1
	    DeSel $tplw $w
	    return $SelLine(coor)
	} elseif { [lindex $com 0] == "ParalleSel" || \
		[lindex $com 0] == "AtomSel"} {
	    DeSel $tplw $w
	    set select(done) 1
	    return
	}
    } else {
	# for SelCentreHoleCLSel its not needed that $nsel == $maxsel
	for {set i 1} {$i <= $nsel} {incr i} {
	    set select(Nat$i) [lindex $select(obj,$i) 2]
	    set select(X$i) [lindex $select(obj,$i) 3]
	    set select(Y$i) [lindex $select(obj,$i) 4]
	    set select(Z$i) [lindex $select(obj,$i) 5]
	}
	if { [lindex $com 0] == "SelCentreHoleCL" } {
	    SelCentreHoleCL
	    return 
	} elseif { [lindex $com 0] == "SelCentreHoleCLSel" } {
	    puts stdout "SELCENTREHOLECLSEL" 
	    set SelHoleCL(done) 1
	    set SelHoleCL(centre) [SelCentreHoleCL]
	    DeSel $tplw $w
	    return $SelHoleCL(centre)
	}
    }
}


proc NextSel {com} {
    global select maxsel nsel text

    # first deselect atoms
    # than deselect line
    #for {set i 1} {$i <= $nsel} {incr i} {
    #	 puts stdout "deselect atomID:: $select(atomID,$i)"
    #	 flush stdout
    #	 xc_deselect .mesa atom $select(atomID,$i)
    #}
    #xc_deselect .mesa line
    #xc_select .mesa finish
    xc_select .mesa clean
    xc_select .mesa begin
    set select(initialized) 1 
    # delete all text
    $text delete 0.0 end 
    # update "iron text"
    $text insert 1.0 \
    "--------------------------------------------------------------------------------\n"
    for {set i 1} {$i <= $maxsel} {incr i} {
	if { $com != "SelCentreHoleCL" } {
	    $text insert [expr $i + 1].0 "Selected Atom No.$i: \n"
	}
    }
    if { $com != "SelCentreHoleCL" } {
	$text insert [expr $maxsel + 2].0 \
	"--------------------------------------------------------------------------------\n\n"
    } 
    set nsel 0
}


#-----------------------------------------------------------------------------
# this is for Line-Adding type of ATOMINSE command
proc LineSel {{w {}}} {
    global select maxsel text SelLine err system
    
    ########################################
    # CD to $system(SCRDIR)
    cd $system(SCRDIR)
    ########################################

    # is fraction of line-length specified correctly
    check_var {{SelLine(fract) fract}} $SelLine(entry)
    if $err { return }
    
    # R = R1 + t*(R2-R1); t...fraction of R2-R1
    set t $SelLine(fract)
    set x [expr $select(X1) + $t*($select(X2) - $select(X1))] 
    set y [expr $select(Y1) + $t*($select(Y2) - $select(Y1))] 
    set z [expr $select(Z1) + $t*($select(Z2) - $select(Z1))] 

    # translate point to first cell
    set fileID [open "$system(SCRDIR)/xc_tmp.$system(PID)" w]
    puts $fileID "0 $x $y $z"
    flush $fileID
    close $fileID
    if [catch {set coor [exec $system(FORDIR)/atomlab 2 \
	    $system(SCRDIR)/xc_struc.$system(PID) \
	    $system(SCRDIR)/xc_tmp.$system(PID)]} errmsg] {
	$text delete [expr $maxsel + 3].0 [expr $maxsel + 4].end
	$text insert [expr $maxsel + 4].0 $errmsg
    } else {
	set xx [lindex $coor 0]
	set yy [lindex $coor 1]
	set zz [lindex $coor 2]
	$text insert [expr $maxsel + 3].0 "Coordinates of point are: \
		[format {%6.10f  %6.10f  %6.10f} $x $y $z]"
	$text insert [expr $maxsel + 4].0 "Coordinates translated to: \
		[format {%6.10f  %6.10f  %6.10f} $xx $yy $zz]"	    
	    return "$xx $yy $zz"
    }
}


# -------------------------------------------------------------------------
# this is for selecting a centre of CLUSTER (AdvGeom option)
# WE NEED TO SELECT ONE ATOM
proc SelCentreAtomCL {{w {}}} {
    global select maxsel text
    $text delete [expr $maxsel + 4].0 [expr $maxsel + 4].end
    $text insert [expr $maxsel + 4].0 \
	    "Coordinates of selected atom are: \
	    $select(X1) $select(Y1) $select(Z1)"
    return "$select(X1) $select(Y1) $select(Z1)"
}


# -------------------------------------------------------------------------
# this is for selecting a centre of CLUSTER (AdvGeom option)
# WE NEED TO SELECT A HOLE (geometrical centre of several atoms)
proc SelCentreHoleCL {{w {}}} {
    global select nsel maxsel text SelHoleCL system
    #nsel ... number of selected atoms

    ########################################
    # CD to $system(SCRDIR)
    cd $system(SCRDIR)
    ########################################
    
    set xc 0.0
    set yc 0.0
    set zc 0.0
    for {set i 1} {$i <= $nsel} {incr i} {
	set xc [expr $xc + $select(X$i)]
	set yc [expr $yc + $select(Y$i)]
	set zc [expr $zc + $select(Z$i)]
    }
    $text delete [expr $nsel + 2].0 [expr $maxsel + 4].end
    $text insert [expr $nsel + 2].0 \
   "--------------------------------------------------------------------------------\n\n"
    # if SetHoleCL(transl) exists --> translate hole to basic cell
    puts stdout "[array names SelHoleCL]"
    if [info exists SelHoleCL(transl)] {
	puts stdout "SetHoleCL(transl)] EXISTS"
	set fileID [open "$system(SCRDIR)/xc_tmp.$system(PID)" w]
	puts $fileID "0 [expr $xc / $nsel] [expr $yc / $nsel] \
		[expr $zc / $nsel]"
	flush $fileID
	close $fileID

	if [catch {set label [exec $system(FORDIR)/atomlab 2 \
		$system(SCRDIR)/xc_struc.$system(PID) \
		$system(SCRDIR)/xc_tmp.$system(PID)]} errmsg] {
	    $text delete [expr $maxsel + 3].0 [expr $maxsel + 4].end
	    $text insert [expr $maxsel + 4].0 $errmsg
	} else {
	    set x [lindex $label 0]
	    set y [lindex $label 1]
	    set z [lindex $label 2]
	    $text insert [expr $maxsel + 3].0 "Coordinates of \"hole\" are: \
		    [format {%6.10f  %6.10f  %6.10f} [expr $xc / $nsel] \
	    [expr $yc / $nsel] [expr $zc / $nsel]]\n"
	    $text insert [expr $maxsel + 4].0 "Coordinates translated to: \
		    [format {%6.10f  %6.10f  %6.10f} $x $y $z]"	    
	    return "$x $y $z"
	}
    } else {
	
    $text insert [expr $nsel + 4].0 \
	    "Coordinates of \"hole\" are: [format {%6.10f  %6.10f  %6.10f} \
	    [expr $xc / $nsel] [expr $yc / $nsel] [expr $zc / $nsel]]"
	return "[expr $xc / $nsel] [expr $yc / $nsel] [expr $zc / $nsel]"
    }
}
    
    
proc Plane {{w {}}} {
    global select maxsel text Plane system


    ########################################
    # CD to $system(SCRDIR)
    cd $system(SCRDIR)
    ########################################

    #----------------------------------------------
    # I THINK THAT $w is NOT NEEDED -CHECK THIS
    # if I gonna remove $w, than it should be also removed from the
    # Angle Dihedral Distance procs & from all calls
    #-----------------------------------------------

    # first check for linear dependence of atomic positions
    set x21 [expr $select(X1) - $select(X2)]
    set y21 [expr $select(Y1) - $select(Y2)]
    set z21 [expr $select(Z1) - $select(Z2)]
    set d21 [expr sqrt( $x21 * $x21 + $y21 * $y21 + $z21 * $z21)] 

    set x32 [expr $select(X2) - $select(X3)]
    set y32 [expr $select(Y2) - $select(Y3)]
    set z32 [expr $select(Z2) - $select(Z3)]
    set d32 [expr sqrt( $x32 * $x32 + $y32 * $y32 + $z32 * $z32)] 

    set ab [expr $x21 * $x32 + $y21 * $y32 + $z21 * $z32]
    set dif [ expr abs($ab - $d21 * $d32) ]
    if { $dif < 0.01 } {
	$text delete [expr $maxsel + 3].0 [expr $maxsel + 4].end
	$text insert [expr $maxsel + 3].0 \
		"WARNING: linear dependence of atomic positions !!!"
	return
    }
     
    # puts the coor of 3 sel. points into $system(SCRDIR)/xc_tmp.$pid
    set plncrd [open "$system(SCRDIR)/xc_tmp.$system(PID)" w]
    puts $plncrd "$select(X1) $select(Y1) $select(Z1)"
    puts $plncrd "$select(X2) $select(Y2) $select(Z2)"
    puts $plncrd "$select(X3) $select(Y3) $select(Z3)"
    flush $plncrd
    close $plncrd
    puts stdout "$select(X1) $select(Y1) $select(Z1)"
    puts stdout "$select(X2) $select(Y2) $select(Z2)"
    puts stdout "$select(X3) $select(Y3) $select(Z3)\n##############"
    eval [list exec cat $system(SCRDIR)/xc_tmp.$system(PID)]
    # now calculate a plane
    xcDebug "$system(FORDIR)/calplane \
	    $system(SCRDIR)/xc_struc.$system(PID) \
	    $system(SCRDIR)/xc_tmp.$system(PID)"

    set fileID [open "|$system(FORDIR)/calplane \
	    $system(SCRDIR)/xc_struc.$system(PID) \
	    $system(SCRDIR)/xc_tmp.$system(PID)"]
    set Plane [read $fileID]
    set Plane1 [lrange $Plane 0 2]
    set Plane2 [lrange $Plane 3 5]
    close $fileID

    $text delete [expr $maxsel + 3].0 [expr $maxsel + 4].end
    $text insert [expr $maxsel + 3].0 \
	    "Unrounded Miller indexes: $Plane1\n"
    $text insert [expr $maxsel + 4].0 \
	    "Miller indexes rounded to: $Plane2"
    
#    $text insert [expr $maxsel + 3].0 "$Plane"
    return $Plane2
}    


proc Dihedral {w} {
    global select maxsel text

    set pi 3.14159265358979323844
    # we've three vectors: 21,32,34
    # vector 32 is normal vector
    set x21 [expr $select(X1) - $select(X2)]
    set y21 [expr $select(Y1) - $select(Y2)]
    set z21 [expr $select(Z1) - $select(Z2)]
    puts stdout "T>>>21: $x21 $y21 $z21"
    # normal vector:
    set x32 [expr $select(X2) - $select(X3)]
    set y32 [expr $select(Y2) - $select(Y3)]
    set z32 [expr $select(Z2) - $select(Z3)]
    puts stdout "T>>>32: $x32 $y32 $z32"
    set x34 [expr $select(X4) - $select(X3)]
    set y34 [expr $select(Y4) - $select(Y3)]
    set z34 [expr $select(Z4) - $select(Z3)]
    puts stdout "T>>>34: $x34 $y34 $z34"
    # now vectror ar...projection of 21 on surface R (n is norm.v. of R)
    #             br...projection of 34 on surface R
    # la -lambda a; factor
    # lb -lambda b; factor
    set sd32 [expr $x32 * $x32 + $y32 * $y32 + $z32 * $z32]
    set la [expr ( $x21 * $x32 + $y21 * $y32 + $z21 * $z32 ) / $sd32 ]
    set lb [expr ( $x34 * $x32 + $y34 * $y32 + $z34 * $z32 ) / $sd32 ]
    puts stdout "T>>>la: $la"
    puts stdout "T>>>lb: $lb"
    set xar [expr $x21 - $la * $x32]
    set yar [expr $y21 - $la * $y32]
    set zar [expr $z21 - $la * $z32]
    set ar [expr sqrt($xar * $xar + $yar * $yar + $zar * $zar)]

    set xbr [expr $x34 - $lb * $x32]
    set ybr [expr $y34 - $lb * $y32]
    set zbr [expr $z34 - $lb * $z32]
    set br [expr sqrt($xbr * $xbr + $ybr * $ybr + $zbr * $zbr)]

    set arbr [expr $ar * $br]

    if { $arbr < 0.01 } {
	$text delete [expr $maxsel + 4].0 [expr $maxsel + 4].end
	$text insert [expr $maxsel + 4].0 \
		"WARNING: Linear dependance of atomic positions !!!"
	return
    }
    set var [expr ( $xar * $xbr + $yar * $ybr + $zar * $zbr ) / \
	    ( $arbr )]
    # $var colud be greater than one, due to roundoff error
    if { $var > 1.0 } {set var 1.0}
    puts stdout "acos:: $var"
    set dihedr [expr acos($var) * 180.0 / $pi]
    $text delete [expr $maxsel + 4].0 [expr $maxsel + 4].end 
    $text insert [expr $maxsel + 4].0 "Dihedral Angle is $dihedr"    
    return $dihedr
}
    
    
proc Angle {w} {
    global select maxsel nsel text

    set pi 3.14159265358979323844
    # this is just for testing
    if { $nsel == $maxsel } {	
	set x21 [expr $select(X1) - $select(X2)]
	set y21 [expr $select(Y1) - $select(Y2)]
	set z21 [expr $select(Z1) - $select(Z2)]
	set dist21 [expr sqrt($x21 * $x21 + $y21 * $y21 + $z21 * $z21)] 
	set x23 [expr $select(X3) - $select(X2)]
	set y23 [expr $select(Y3) - $select(Y2)]
	set z23 [expr $select(Z3) - $select(Z2)]
	set dist23 [expr sqrt($x23 * $x23 + $y23 * $y23 + $z23 * $z23)]
	set var [expr ( $x21 * $x23 + $y21 * $y23 +  $z21 * $z23 ) / \
		( $dist21 * $dist23 )]
	# var could be greater than 1.0, due to roundoff error
	if { $var > 1.0 } {set var 1.0}
	set angl [expr acos($var) * 180.0 / $pi]
	$text delete [expr $maxsel + 4].0 [expr $maxsel + 4].end 
	$text insert [expr $maxsel + 4].0 "Angle is $angl"    
	return $angl
    }
}


proc Distance {w} {
    global select maxsel nsel text

    # this is just for testing
    if { $nsel == $maxsel } {
	puts stdout "DISTANCE BETWEEN ATOMS: \n\
		$select(X1) $select(Y1) $select(Z1) \n\
		$select(X2) $select(Y2) $select(Z2)"
	set dx [expr $select(X1) - $select(X2)]
	set dy [expr $select(Y1) - $select(Y2)]
	set dz [expr $select(Z1) - $select(Z2)]
	set dist [expr sqrt($dx * $dx + $dy * $dy + $dz * $dz)] 
	# first delete, then insert
	$text delete [expr $maxsel + 4].0 [expr $maxsel + 4].end 
	$text insert [expr $maxsel + 4].0 "Distance is $dist"    
	return $dist
    }
}
    

proc AtomInfo {{w {}}} {
    # this is dummy procedure
    return 
}


proc SelectAtom { x y } {
    global select nsel maxsel text Xor Yor df zoom

    xcDebug -debug "SelectAtom"
    # if we click the atom first time --> SELECTION
    # if we click the atom second time --> DESELECTION
    #set tx [lindex [split $text .] 1]
    #raise .$tx .
    #update

    set sel 1
    set obj [xc_select .mesa sqn $x $y]
    # if $obj == 0 -> no atom was selected
    if { $obj == 0 } { 
	xcDebug -debug "AtomSelect End"
	return 
    }

    for {set i 1} {$i <= $nsel} {incr i} {
	if { $obj == $select(atomID,$i) } {
	    # deselect atom
	    xc_deselect .mesa atom $obj
	    $text delete [expr $i + 1].20 [expr $i + 1].end
	    
	    set sel 0
	    for {set j $i} {$j < $nsel} {incr j} {
		set jj [expr $j + 1]
		$text delete [expr $jj + 1].20 [expr $jj + 1].end
		$text insert [expr $j + 1].20 $select(obj,$jj)
		set select(atomID,$j) $select(atomID,$jj)
		set select(obj,$j) $select(obj,$jj)
	    }
	    if { $maxsel > 4 } {
		# "SelCentreHoleCL" 
		$text delete [expr $nsel + 1].0 [expr $nsel + 1].end 
	    }

	    set nsel [expr $nsel - 1]		   		
	    
	    # THIS IS FOR LINES REARRANGEING
	    puts stdout "THIS IS FOR LINES REARRANGEING:: nsel=$nsel"
	    if { $nsel >= 1 } { xc_deselect .mesa line }
	    for {set j 1} {$j < $nsel} {incr j} {
		set jj [expr $j + 1]		
		set select(lineID,$j) [xc_select .mesa line \
			$select(atomID,$j) $select(atomID,$jj)]
	    }
	    # finish changes
	    xc_select .mesa finish
	    return
	}
    }
    	
    if { $nsel < $maxsel && $sel == 1} {
	incr nsel
	set select(atomID,$nsel) $obj	
	puts stdout "atomID: $obj"
	set select(obj,$nsel) [xc_select .mesa atom $obj]
	if { $maxsel > 4 } {
	    # "SelCentreHoleCL" 
	    $text insert [expr $nsel + 1].0 "Selected Atom No.$i: \n"
	}
	$text insert [expr $nsel + 1].20 $select(obj,$nsel)
    }
    # DRAW A LINE BETWEEN SELECT ATOM & PREVIOUS SELECTED ATOM
    if { $nsel <= $maxsel && $nsel > 1 } {
	set nsel1 [expr $nsel - 1]
	set select(lineID,$nsel1) [xc_select .mesa line \
		$select(atomID,$nsel1) $select(atomID,$nsel)]
    }

    # now flush to display all changes
    xc_select .mesa finish
    xcDebug -debug "AtomSelect End"
    update
}

proc PreSel {w c topname title com m} {
    global maxsel mode nsel text SelLine select

    # PreSel ...PrepareSelection
    # w ...     name of topwidget
    # c ...     name of canvas
    # topname . name of toplevel
    # title ... title 
    # com ..... command to exexute
    # m ....... set maxsel $m

    set select(selection_mode) 1

    # some INITIALIZATIONS    
    $c config -cursor cross
    bind $c <Button-1> { SelectAtom %x %y }
    # due to configuring of $c the display of $c vanish, 
    # to fix that call "update"
    update

    xc_select .mesa begin
    set select(initialized)    1
    set nsel 0
    set maxsel $m
    
    # make TOPWIDGET
    if [winfo exists $w] { return }
    xcToplevel $w $topname "Selection" . 0 0 1
    AlwaysOnTopON . $w
    bind $w <Destroy> [list DeSel $w $c]
    focus $w

    set topfrm [frame $w.topfrm -class StressText]
    set titl [label $topfrm.lbl -text $title -relief groove -bd 2]
    pack $topfrm -side top -expand 1 -fill x 
    pack $titl -expand 1 -fill x -ipady 3 -ipadx 10 -pady 10 -padx 10
    #set frm [frame $w.frm -bd 1]
    if { $maxsel <= 4 } {
	set height [expr $maxsel + 4]
	#set text [text $w.text -relief sunken -width 80 -height \
	#	$height -font fixed]    
	set text [text $w.text -relief sunken \
		-width 80 -height $height -font fixed]
    } else {
	# for SelCentreHoleCL maxsel could be up to 15
	set height 8
	set fts [frame $w.f1]
	pack $fts -side top -expand 1
	set text [text $fts.text -relief sunken -width 80 -height \
		$height -yscrollcommand "$fts.sy set" -font fixed]
	scrollbar $fts.sy -orient vert -command "$text yview" 
	pack $fts.sy -side right -fill y
    }
    set botfrm [frame $w.botfrm]

    # if $comm == "LineSel" -> there must also be entry for specifying the 
    # line-length fraction
    if { $com == "LineSel" } {
	set llf [frame $botfrm.llf -relief groove -bd 2]
	pack $llf -side left -expand 1 -padx 20 -pady 10
	Entries $llf {{Line-Length Fraction:}} SelLine(fract) 10
	# remember which entry to focus in case of error
	set SelLine(entry) $llf.frame.entry1
    }
    
    if { $com != "AtomInfo" && $com != "AtomSel" && $com != "ParalleSel" } {
	set but1 [button $botfrm.done -text "Done" -command \
		[list Done $c "$com $c"]]
	pack $but1 -side left -padx 5 -pady 3 -expand 1
    }
    
    set but2 [button $botfrm.next -text "Next" \
	    -command [list NextSel $com]]
    set but3 [button $botfrm.can -text "Close" -command \
	    [list DeSel $w $c]]

    pack $text -side top -padx 10 -pady 3 -fill x -expand 1    
    pack $botfrm -side top -fill both -expand 1
    pack $but2 $but3 -side left -padx 5 -pady 3 -expand 1

    # extra button when we are selecting a Plane or "Centre of Cluster"
    # for AdvGeom
    if { $com == "PlaneSel" || $com == "SelCentreAtomCL" || \
	    $com == "SelCentreHoleCL" || $com == "LabelSel" || \
	    $com == "LineSel" } {
	set but4 [button $botfrm.sel -text "Selected" -command \
		[list Done $c "${com}Sel $c" $w]]
	pack $but4 -side left -padx 5 -pady 3 -expand 1
    }
    
    # extra button when we are selecting the IsoSpace/IsoPlane
    if { $com == "AtomSel" || $com == "ParalleSel" } {
	set but4 [button $botfrm.sel -text "Selected" -command \
		[list Done $c ${com} $w]]
	pack $but4 -side left -padx 5 -pady 3 -expand 1
    }
    
    $text insert 1.0 \
    "--------------------------------------------------------------------------------\n"
    for {set i 1} {$i <= $maxsel} {incr i} {
	if { $com != "SelCentreHoleCL" } {
	    $text insert [expr $i + 1].0 "Selected Atom No.$i: \n"
	}
    }
    # if $com == "SelCentreHoleCL", we do not know how many atoms will be 
    # selected, but for all other $com we do know
    if { $com != "SelCentreHoleCL" } {
	$text insert [expr $maxsel + 2].0 \
	"--------------------------------------------------------------------------------\n\n"
    }

    return $w
}


proc DeSel {w c} {
    global xcCursor select
    
    xcDebug "DeSel"
    if $select(initialized) {
	# in order to clean, initialization should be performed first !!!
	xc_select .mesa clean
    }
    set select(selection_mode) 0
    set select(initialized)    0
    
    $c config -cursor $xcCursor(default)
    bind $c <Button-1> {}

    if [winfo exists $w] { 
	# this destroy will cause that DeSel will be entered few 
	# times (recursively), but the DeSel is writen good enough 
	# to handle that
	destroy $w 
    }
}
