#############################################################################
# Author:                                                                   #
# ------                                                                    #
#  Anton Kokalj                                  Email: Tone.Kokalj@ijs.si  #
#  Department of Physical and Organic Chemistry  Phone: x 386 1 477 3523    #
#  Jozef Stefan Institute                          Fax: x 386 1 477 3811    #
#  Jamova 39, SI-1000 Ljubljana                                             #
#  SLOVENIA                                                                 #
#                                                                           #
# Source: $XCRYSDEN_TOPDIR/Tcl/sizes.tcl                                       
# ------                                                                    #
# Copyright (c) 1996-2003 by Anton Kokalj                                   #
#############################################################################

proc ProbeResolution {} {
    global xcMisc xcFonts

    if [catch {exec xdpyinfo} xinfo] {
	tk_dialog [WidgetName] ERROR "ERROR: An error occured while probing the screen resolution with xdpyinfo program. Assuming 1024x768 scren-resolution." error 0 OK
	set xcMisc(resolution)  1024x768
	set xcMisc(resolutionX) 1024
	set xcMisc(resolutionY) 768
	set xcMisc(resolution_ratio1) 1.0
	set xcMisc(resolution_ratio2) 1.0
    } else {
	foreach line [split $xinfo \n] {
	    if [string match *dimensions:* $line] {
		set xcMisc(resolution)  [lindex $line 1]
		set xcMisc(resolutionX) \
			[lindex [split $xcMisc(resolution) x] 0]
		set xcMisc(resolutionY) \
			[lindex [split $xcMisc(resolution) x] 1]
		#xcDebug "resX: $xcMisc(resolutionX)"
		break
	    }
	}
    }

    #
    # resolution >= 1024x768 -> takes tk's defaults
    # resolution [800x600,1024x768] -> small size = 8; normal size = 10;
    # resolution [600x480,800x600]  -> small size = 7; normal size = 8;
    #
    #set xcMisc(resolutionX) 600; # debugging only
    set xcFonts(small_size)  8
    set xcFonts(normal_size) 9
    set xcMisc(resolution_ratio1) 0.65
    set xcMisc(resolution_ratio2) 0.75
    if { $xcMisc(resolutionX) > 700 } {
	set xcFonts(small_size)  8
	set xcFonts(normal_size) 10
	set xcMisc(resolution_ratio1) 0.75
	set xcMisc(resolution_ratio2) 0.85
    } 
    if { $xcMisc(resolutionX) > 1000 } {
	set xcFonts(small_size)  10
	set xcFonts(normal_size) 12
	set xcMisc(resolution_ratio1) 1.0
	set xcMisc(resolution_ratio2) 1.0
    }

    #
    # now set the default fonts
    #
    foreach widclass {{button Button} {checkbutton Checkbutton} {entry Entry} \
	    {label Label} {listbox Listbox} {menu Menu} \
	    {menubutton Menubutton} \
	    {message Message} {radiobutton Radiobutton} {scale Scale} \
	    {text Text}} {
	set widget [lindex $widclass 0]
	set class  [lindex $widclass 1]
	$widget .test_widget
	set font \
		[ModifyFont [.test_widget cget -font] .test_widget \
		-size $xcFonts(normal_size) -default 1]
	#set font \
	#	[ModifyFontSize .test_widget $xcFonts(normal_size)]

	if { $xcMisc(resolutionX) < 1000 } {
	    ###########################################
	    option add *$class.font $font widgetDefault
	    ###########################################
	}

	if { $widget == "button" } {
	    set xcFonts(normal) $font
	    set xcFonts(small) \
		    [ModifyFont [.test_widget cget -font] .test_widget \
		    -size $xcFonts(small_size) -default 1]
	} elseif { $widget == "entry" } {
	    set xcFonts(normal_entry) $font
	    set xcFonts(small_entry) \
		    [ModifyFont [.test_widget cget -font] .test_widget \
		    -size $xcFonts(small_size) -default 1]
	}
	destroy .test_widget
    }
    xcDebug "Taking settings for $xcMisc(resolution) screen-resolution !!!"
    xcDebug "-------------------"
    xcDebug "Small  Font Size set to:   $xcFonts(small_size)"
    xcDebug "Normal Font Size set to:   $xcFonts(normal_size)"
    #xcDebug "Big    Font Size set to:"
}

proc SetImageSizes {} {
    global xcMisc

    foreach image [image names] {
	set h [image height $image]
	set w [image width  $image]
	#xcDebug "h=$h"
	set f $xcMisc(resolution_ratio2)
	if { $f < 0.75 } { set f 0.75 }
	set h [expr round( $h * $f )]
	set w [expr round( $w * $f )]
	$image configure -height $h -width $w
	# debugging only
	#xcDebug "Image dim:: ${w}x$h"
	#button .$image -image $image -anchor center
	#pack .$image -side left
    }
}
