#!/bin/sh

if [ $# -eq 0 ]; then
    input=-
elif [ $# -eq 1 ]; then
    input=$1
else
    echo "Usage: 
    g98toxsf.sh G98-output > XSF-file
or
    g98toxsf.sh < G98-output > XSF-file"
    exit 1
fi


#--------------------------------------------------------------------------
# this is an experimental Gaussian98 To XSF converter. Use on your own risk
#
# Usage:   g98toxsf.sh G98-output > XSF-file
#        or
#          g98toxsf.sh < G98-output > XSF-file
#
# these are the records that we look for:
# 
# 0                          Input orientation:                          
# 1 ---------------------------------------------------------------------
# 2 Center     Atomic     Atomic              Coordinates (Angstroms)
# 3 Number     Number      Type              X           Y           Z
# 4 ---------------------------------------------------------------------
# 5    1          8             0         .000000     .000000     .000000
# 6    2          1             0         .000000     .000000    1.000000
# 7    3          1             0         .968148     .000000    -.250380
# 8 ---------------------------------------------------------------------
# 
# 
# 0 Center     Atomic                   Forces (Hartrees/Bohr)
# 1 Number     Number              X              Y              Z
# 2 -------------------------------------------------------------------
# 3    1          8           -.051649003     .000000000    -.039990931
# 4    2          1            .017695138     .000000000     .030494684
# 5    3          1            .033953865     .000000000     .009496247
# 6 -------------------------------------------------------------------
#
#------------------------------------------------------------------------
cat $input | awk '
BEGIN { 
  is_opt = 0; 
  command_deck = 0;
}
{ 
  if ( $1 == "#" && command_deck == 0) {
    if ( toupper($0) ~ /OPT/ ) {
      is_opt = 1;
    }
    command_deck = 1;
  }  
}
{
  if ( $0 ~ / +Input orientation:/ ) {
    i=0;
    getline;
    getline;
    getline;
    getline;
    getline;
    printf " %s\n", "ATOMS"; 
    while ( $1 !~ /\-\-/ ) {
      #print "COOR:", $0;
      if ( ! is_opt ) {
	printf "%3d   %15.10f %15.10f %15.10f\n", $2, $4, $5, $6;
      } else {
	atn[i] = $2;
	x[i]   = $4;
	y[i]   = $5;
	z[i++] = $6;
      }
      getline;
    }
  }
}
{
  if ( $0 ~ /Forces \(Hartrees\/Bohr\)/ ) {
    #print "LINE:", $0;
    i=0;
    getline;
    getline;
    getline;
    while ( $1 !~ /\-\-/ && is_opt ) {
      printf "%3d   %15.10f %15.10f %15.10f   %15.10f %15.10f %15.10f\n", 
	atn[i], x[i], y[i], z[i], $3, $4, $5;
      i++;
      getline;
    }
  }
}
END {
  if ( is_opt ) {
# print OPTIMIZED coordinates
    for (ia=0; ia<i; ia++) 
      printf "%3d   %15.10f %15.10f %15.10f   %15.10f %15.10f %15.10f\n", 
	atn[ia], x[ia], y[ia], z[ia], 0.0, 0.0, 0.0;
  }
}' > g98toxsf.$$

nstep=$(grep ATOMS g98toxsf.$$ | wc | awk '{print $1}')
if [ $nstep -gt 1 ]; then
# make a AXSF file
    cat g98toxsf.$$ | awk -v ns=$nstep '
BEGIN {
  printf " %s %d\n", "ANIMSTEPS", ns;
}
/ATOMS/ { printf " %s %d\n", "ATOMS", ++i; next; }
/a*/    { print; }'
else
    cat g98toxsf.$$
fi

rm -f g98toxsf.$$
exit 0


    
