#!/bin/sh

#
# pwi2xsf.sh: PW-input to XSF converison
#
# Usage: pwi2xsf [-r] pw-input-file
#
# Written by Tone Kokalj on Tue May  8 20:43:44 2001
#            ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

if [ "$#" -lt 1 ]; then
    echo "
Usage: pwi2xsf.sh [-r] pw-input

-r ... one must spefify i.e. ityp->nat conversion, and the corresponding
       data are writen to file nuclei.charges. The -r flag deletes this
       file.
"
    exit 1
fi

r=0
if [ "$1" = "-r" ]; then
    r=1
    shift
fi

#######################################
if [ "$XCRYSDEN_TOPDIR" != "" ]; then
    . $XCRYSDEN_TOPDIR/scripts/pwLib.sh
else
    . pwLib.sh
fi
#######################################

#
# check if we have OLD or NEW PW.X input format
#
new_format1=$(grep 'ATOMIC_POSITIONS' $1)
new_format2=$(grep -i '&system' $1)

if [ "$new_format1" != ""  -a  "$new_format2" != "" ]; then
    # we have NEW PW.X input format
    #
cat $1 | awk 'BEGIN {RS=",";} {print $0}' | awk '
BEGIN {
  end=0;
}

/&system/          { sysnml=1; print; }
/ibrav|celldm|nat/ { print; }

/ATOMIC_SPECIES|ATOMIC_POSITIONS|K_POINTS|CELL_PARAMETERS/ {
  print_line=1;
  print toupper($0);
  next;
}

/&end|^\/|^ \// { 
  if ( sysnml == 1 ) { 
    print; 
    sysnml=0; 
  } 
}

/a*/ {
  if ( print_line == 1 ) {
    print toupper($0);
  }
}'> pw.$$
    PWI2XSF=pwi2xsf_new
else 
    # we have OLD PW.X input format	

    pwNucleiCharges $1 /dev/null

    cat $1 | awk 'BEGIN {RS=",";} {print}' | awk '
BEGIN {
    end=0;
}
/&input|celldm|nat|ltaucry/ { print; }
/ibrav/ { 
    print;
    split($0,a,"=");
    split(a[1],b,",");
    ibrav = b[1];
}
/&end|^\/|^ \// { end=1; }
/a*/ {
    if ( end == 1 ) print;
}' > pw.$$
    PWI2XSF=pwi2xsf
fi
#
# execute $PWI2XSF fortran program and print the XSF file
#
if [ "$XCRYSDEN_TOPDIR" != "" ]; then
    $XCRYSDEN_TOPDIR/bin/$PWI2XSF < pw.$$ | tee pwi2xsf.xsf_out
else
    $PWI2XSF < pw.$$ | tee pwi2xsf.xsf_out
fi
rm -f pw.$$

if [ "$r" -eq 1 ]; then
    rm nuclei.charges
fi

exit 0