# ------------------------------------------------------------------------
#   FUNCTIONS
# ------------------------------------------------------------------------

CustomDir () {
    echo -n "
   Do you want to create \$HOME/.xcrysden directory and copy XCRYSDEN 
   definition files to it. By  doing that you will be able to do some 
   customization for XCRYSDEN ([y]es/[n]o): " 

    read ans
    if [ \( "$ans" != "y" \) -a \( "$ans" != "n" \) ]; then
	echo "
Please answer y or n !!!"
	CustomDir
    fi
}

ExeWarning () {
    if [ ! -x "$1" ]; then
	echo "WARNING: file $1 is not executable !!!"
    fi
}

QueryProgram() {
    # usage: QueryProgram msg1 mesg2 deafult variable
    echo -n "$1 ([y]es/[n]o): "
    read ans; 
    if [ \( "$ans" != "y" \) -a \( "$ans" != "n" \) ]; then
	echo "Please answer y or n !!!"
	QueryProgram "$1" "$2" "$3" "$4"
    fi
    if [ "$ans" = "y" ]; then
	echo -n "$2 (default: $3): "
	read ans;
	if [ "$ans" = "" ]; then
	    ans=$3
	fi
	ExeWarning $ans
	eval $4=$ans
    else
	eval $4=""
    fi
}

CustomDef () {
    echo -n "Do you have a CRYSTAL package ([y]es/[n]o): "
    read cXX
    echo "" > $tempdir/customFile.$$
    if [ \( "$cXX" != "y" \) -a \( "$cXX" != "n" \) ]; then
	echo "Please answer y or n !!!"
	CustomDef
    fi
    if [ "$cXX" = "y" ]; then	
	echo -n "Specify CRYSTAL integrals module (default: /usr/local/bin/integrals):  "
	read int
	if [ "$int" = "" ]; then
	    int=/usr/local/bin/integrals
	fi
	ExeWarning $int

	echo -n "Specify CRYSTAL scf module (default: /usr/local/bin/scf):  "
	read scf
	if [ "$scf" = "" ]; then
	    scf=/usr/local/bin/scf
	fi
	ExeWarning $scf

	echo -n "Specify CRYSTAL properties module (default: /usr/local/bin/properties):  "
	read pro
	if [ "$pro" = "" ]; then
	    pro=/usr/local/bin/properties
	fi
	ExeWarning $pro
	echo "
# ------------------------------------------------------------------------
#  do we have CRYSTAL package
# ------------------------------------------------------------------------

set system(c95_exist) 1


# ------------------------------------------------------------------------
#  CRYSTAL modules
# ------------------------------------------------------------------------

set system(c95_integrals)  $int
set system(c95_scf)        $scf
set system(c95_properties) $pro

" >> $tempdir/customFile.$$
    else
	echo "
# ------------------------------------------------------------------------
#  do we have CRYSTAL package
# ---------------------------

set system(c95_exist) 0


# ------------------------------------------------------------------------
#  CRYSTAL modules
# ------------------------------------------------------------------------

#set system(c95_integrals)  /full-path-to/integrals
#set system(c95_scf)        /full-path-to/scf
#set system(c95_properties) /full-path-to/properties

" >> $tempdir/customFile.$$
    fi

    # ------------------------------------------------------------------------
    #  Query PRINT command
    # ------------------------------------------------------------------------
    echo -n "Specify print command (default: lpr -h):  "
    read lpr 
    if [ "$lpr" = "" ]; then
	lpr="lpr -h"
    fi

    
    echo "

   Please answer to the following answers questions below. Here is a
   short description of the questions:

   1.) By specifying an image conversion program will make XCRYSDEN able to
       print-to-file in JPG/PNG/GIF and other formats. The \"convert\" 
       program of ImageMagick (http://www.imagemagick.org/) is a convenient 
       choice.

   2.) If you have a \"whirlgif\" program then XCRYSDEN will be able
       to create an Animated-GIF images from the animation snapshots.

   3.) If you have an \"mpeg_encode\" program then XCRYSDEN will be able
       to create an MPEG movie from the animation snapshots.

" | more

    # ------------------------------------------------------------------------
    #  Query image conversion program
    # ------------------------------------------------------------------------
    QueryProgram "Do you want to enable image conversion program" \
	"Which image conversion program" /usr/bin/convert convert

    # ------------------------------------------------------------------------
    #  Query Whirlgif program
    # ------------------------------------------------------------------------
    QueryProgram "Do you want to enable whirlgif program" \
	"Which whirlgif program" /usr/bin/whirlgif whirlgif

    # ------------------------------------------------------------------------
    #  Query mpeg_encode program
    # ------------------------------------------------------------------------
    QueryProgram "Do you want to enable mpeg_encode program" \
	"Which mpeg_encode program" /usr/bin/mpeg_encode mpeg_encode 
        
    if [ "$convert" != "" ]; then
	_convert="set xcMisc(ImageMagick.convert) $convert"
    else
	_convert="#set xcMisc(ImageMagick.convert) /full-path-to/convert"
    fi

    if [ "$whirlgif" != "" ]; then
	_whirlgif="set xcMisc(whirlgif) $whirlgif"
    else
	_whirlgif="#set xcMisc(whirlgif) /full-path-to/whirlgif"
    fi

    if [ "$mpeg_encode" != "" ]; then
	_mpeg_encode="set xcMisc(mpeg_encode) $mpeg_encode"
    else
	_mpeg_encode="#set xcMisc(mpeg_encode) /full-path-to/mpeg_encode"
    fi

    echo "

# ------------------------------------------------------------------------
#  print command
# ------------------------------------------------------------------------

set printCanvas(printCommand) {$lpr}


# ------------------------------------------------------------------------
#  An image conversion program: we need PPM to PNG/JPG/GIF/... conversion.
#  The \"convert\" program of ImageMagick (http://www.imagemagick.org/) is
#  a convinient choice.
# ------------------------------------------------------------------------

$_convert 


# ------------------------------------------------------------------------
#  With \"whirlgif\" program XCRYSDEN can create an Animated-GIF image
# ------------------------------------------------------------------------

$_whirlgif


# ------------------------------------------------------------------------
#  MPEG_ENCODE for creating the MPEG movies
# ------------------------------------------------------------------------

$_mpeg_encode


# ------------------------------------------------------------------------
#  Here go custom user-specified options
# ------------------------------------------------------------------------
#  In order to add an --unknown option to the Xcrysden allowed options,
#  do the following
#  
# Usage: 
#       addOption option converterProgram description
#
# Arguments:
#       option           ... option to add to XCRYSDEN options
#       converterProgram ... program that converts from an \"unknown\" to XSF format;
#                            this program must be supplied by the user !!!
#       description      ... description of the options that will appear in the
#                            help message (i.e. xcrysden --help).
# Example: 
#       addOption --unknown /home/tone/utils/unknown2xsf {
#               load structure from unknown file format
#       }
#
" >> $tempdir/customFile.$$

    echo "

 ##############################
  You specified the following:
 ##############################

Do you have a CRYSTAL package: $cXX"
    if [ "$cXX" = "y" ]; then
	echo "
CRYSTAL integrals module:  $int
CRYSTAL scf module:        $scf
CRYSTAL properties module: $pro"
    fi
    echo "Print Command:                 $lpr"
    if [ "$convert"     != "" ]; then echo "Image converting program:      $convert"; fi
    if [ "$whirlgif"    != "" ]; then echo "Whirlgif program:              $whirlgif"; fi
    if [ "$mpeg_encode" != "" ]; then echo "Mpeg_encode program:           $mpeg_encode"; fi

    echo -n "
Is this correct ([y]es/[n]o): "
    read cor
    if [ \( "$cor" != "y" \) -a \( "$cor" != "yes" \) ]; then
	CustomDef
    fi


    echo -n "
Would you line to edit the ~/.xcrysden/custom-definitions file now ([y]es/[n]o):"
    read edit
    if [ \( "$edit" = "y" \) -o \( "$edit" = "yes" \) ]; then
	${EDITOR:-vi} $HOME/.xcrysden/custom-definitions
    fi
}


# ------------------------------------------------------------------------
#   MAIN
# ------------------------------------------------------------------------

if [ \( -d /tmp \) -a \( -w /tmp \) ]; then
    tempdir=/tmp
else
    # set tempdir to $HOME
    tempdir=$HOME
fi

echo "
   ------------------------------------------------------------------------
     TODO:            Please answer to following QUESTIONS!!!
   ------------------------------------------------------------------------
"

CustomDir

if [ "$ans" = "y" ]; then
    if [ ! -d $HOME/.xcrysden ]; then
    	mkdir $HOME/.xcrysden
    fi
    CustomDef
    echo -n "creating $HOME/.xcrysden/custom-definitions ... "
    cp $tempdir/customFile.$$ $HOME/.xcrysden/custom-definitions
    if [ $? -eq 0 ]; then
	echo OK
    else
	echo "
ERROR: failed to create $HOME/.xcrysden/custom-definitions file !!!"
    fi
    echo -n "copying $XCRYSDEN_TOPDIR/Tcl/Xcrysden_defaults ... "
    cp $XCRYSDEN_TOPDIR/Tcl/Xcrysden_defaults $HOME/.xcrysden/Xcrysden_defaults
    if [ $? -eq 0 ]; then
	echo OK
    else
	echo "
ERROR: failed to copy $XCRYSDEN_TOPDIR/Xcrysden_defaults file !!!"
    fi
fi

if [ -f $tempdir/customFile.$$ ]; then
    rm -f $tempdir/customFile.$$
fi

echo "

   *************
   Final NOTE!!!
   *************

   If everything was OK during user-installation, then just source
   your profile. If you are using csh-like shell also apply rehash
   command.

   If you were instructed to manually edit the profile, please edit
   your profile first and then perform the above instructions.

   After that you are able to run XCRYSDEN. Just type: xcrysden

   Please report BUGS to Tone.Kokalj@ijs.si. 

   IMPORTANT NOTICE:
   -----------------
   You should make a reference to XCRYSDEN program whenever you use
   the graphics that were created by the program (applying to
   publications, presentation-slides, conference-posters, and alike).

   XCRYSDEN reference:

    A. Kokalj: J. Mol. Graphics Modelling, 1999, Vol. 17, 176-179.

    A. Kokalj and M. Causa, XCrySDen: X-window CRYstalline Structures 
    and DENsities, http://www-k3.ijs.si/kokalj/xc/, 2002.


    Good Luck !!! 

    Hint: try \"xcrysden --help\"
" | more
